#include "SymbianApp.h"
#include <stdlib.h>
#include <e32keys.h>
#include <es_sock.h> // Basic socket
#include <in_sock.h> // IP sockets
#include <bt_sock.h> // Bluetooth sockets
#include <coecntrl.h>
TInt gOffset = 0; // offset into screen buffer
TInt gDrawnWidth = 0;
TInt gAddition = 0;
TRect gUpdateRect;
TInt gBufferOffset = 0;
TInt gNextLine = 0;
TSize gScreenSize;
#ifdef UIQ3
#include <devicekeys.h>
#include <qikviewbase.h>
const TUint KCharKeyToNumericTable[9][2]=
{
	{'E','R'},
	{'T','Y'},
	{'U','I'},
	{'D','F'},
	{'G','H'},
	{'J','K'},
	{'X','C'},
	{'V','B'},
	{'N','M'}
};

TUint RemapScanCode(TInt aScanCode)
{
	if(aScanCode>= 'B'  && aScanCode < 'Z')
	{
		for(TInt loop = 0;loop<9;loop++)
		{
			if(aScanCode == KCharKeyToNumericTable[loop][0] || aScanCode == KCharKeyToNumericTable[loop][1])
			{
				return '1'+loop;
			}
		}
	}

	return aScanCode;
}

#endif

class CActiveWaiter:public CActive
{
public:
	CActiveWaiter(TInt& aResult):CActive(0), iResult(aResult)
	{
		CActiveScheduler::Add(this);
		iStatus = KRequestPending;	
	}

	~CActiveWaiter()
	{
		CActiveScheduler::Stop();
	}

	void DoCancel()
	{
	}

	void RunL()
	{
		iResult = iStatus.Int();
		delete this;
	}

	void Start()
	{
		SetActive();
		CActiveScheduler::Start();
	}
private:
	TInt& iResult;
};


extern "C" 
{
#include "svga.h"
#include "driver.h"
#include "symbian-runtime.h"
#ifndef __WINS__
#include "asmutils.h"
#endif
void log(int val)
{
	RDebug::Print(_L("mouseLine:%d\n"), val);
}
}

class CGpfceEmulator:
#ifdef UIQ3
public CQikViewBase,
#else
public CCoeControl,
#endif
public MMdaAudioOutputStreamCallback,public MDirectScreenAccess
{
public:
	static CGpfceEmulator* NewL(CGpfceAppUi& aAppUi);
	~CGpfceEmulator();
	void UpdateInputL();
	void WriteSoundL(TInt16* aBuffer, TInt aCount);
	void SelectRomL();
	TBool iForeground;

	void Restart(RDirectScreenAccess::TTerminationReasons aReason);
	void AbortNow(RDirectScreenAccess::TTerminationReasons aReason);
	void UpdateScreenParamsL();
	void UpdateScreenL();
	void ClearScreen();

	void GetKeyStatusL(TUint16& aPlayer1, TUint16& aPlayer2);
	void GetMouseStatusL(TUint8& aX, TUint8& aY);
	void UpdateSampleRateL();

	TInt NetworkConnectL();
	TInt NetworkRecvDataL(TUint8 *data, TUint32 len, TInt block);
	TInt NetworkSendDataL(TUint8 *Value, TUint32 len);
	void NetworkCloseL();
protected:
#ifdef UIQ3
	/** 
	Returns a @c TVwsViewId uniquely identifying the view.
	A view ID includes both an application UID and a specific view UID.
	The view UID, the second UID, needs only to be unique in the application.
	@return A unique identifier for the view.
    */
	TVwsViewId ViewId() const
	{
		return TVwsViewId(TUid::Uid(0xA30033A3), TUid::Uid(1));
	}

	void ViewActivatedL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,const TDesC8& aCustomMessage)
	{
		CQikViewBase::ViewActivatedL(aPrevViewId, aCustomMessageId, aCustomMessage);
		MakeVisible(ETrue);
	}

	void ViewDeactivated()
	{
		CQikViewBase::ViewDeactivated();
		MakeVisible(EFalse);
	}

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
	{
		CQikViewBase::OfferKeyEventL(aKeyEvent, aType);
		return EKeyWasNotConsumed;
	}

	void HandlePointerEventL(const TPointerEvent& aPointerEvent)
	{
		
		switch(Settings.orientation)
		{
		case 0:
			iLastPoint = TPoint(((aPointerEvent.iPosition.iX-gUpdateRect.iTl.iX)+((256-gDrawnWidth)/2))-8, aPointerEvent.iPosition.iY-gUpdateRect.iTl.iY);
			break;
		case 1:
			iLastPoint = TPoint(aPointerEvent.iPosition.iY, aPointerEvent.iPosition.iX);			
			break;
		case 2:
			iLastPoint = TPoint(aPointerEvent.iPosition.iY, aPointerEvent.iPosition.iX);
			break;
		}
		
		
		if(Settings.stretch)
		{
		}
	}
#endif
	CGpfceEmulator(CGpfceAppUi& aAppUi);
	void ConstructL();
	void StartEmulatorL();
	void Draw(const TRect& aRect) const;

	static TInt AsynchStartupL(TAny* aEmuClass);
	static TInt IdleCallBackStop(TAny* aAppUi);

	void MaoscOpenComplete(TInt aError);
	void MaoscBufferCopied(TInt aError, const TDesC8& aBuffer);
	void MaoscPlayComplete(TInt aError);

private:
	CEikonEnv* iEikEnv;
	CDirectScreenAccess* iDsa;

	TBool iDrawingOn;
	CFbsBitmap* iBackBuffer;

	CAsyncCallBack* iAsynchStart;
	CAsyncCallBack* iIdleCallBack;

	CGpfceAppUi& iAppUi;

	TPoint iLastPoint;

	// sound support
	CMdaAudioOutputStream* iSndStream;
	TMdaAudioDataSettings iAudioSettings;
	TBuf8<10000> iAudioBuffer[2];
	TInt iBufferCnt;
	TInt iCurrentBuffer;
	TInt iNewSampleRate;

	// Network support
	RSocketServ iSocketServ;
	RSocket   iSocket;
	RSocket   iListeningSocket;
	TInetAddr iIpAddress;
	TBTSockAddr iBTAddress;
};

CApaApplication* NewApplication() {
	// Return pointer to newly created CQMApp
	return new CGpfceApp;
}

#include <eikstart.h>
// E32Main() contains the program's start up code, the entry point for an EXE.
GLDEF_C TInt E32Main() {
 	return EikStart::RunApplication(NewApplication);
}

CGpfceApp::CGpfceApp()
{
}

CGpfceApp::~CGpfceApp()
{
}

CApaDocument* CGpfceApp::CreateDocumentL()
{
	CGpfceDoc* doc = new (ELeave) CGpfceDoc(*this);
	return doc;
}

CGpfceDoc::CGpfceDoc(CEikApplication& aApp):
#ifdef UIQ3
CQikDocument(aApp)
#else
CAknDocument(aApp)
#endif
{
}

extern "C" 
{
extern char lastLoadedGameName[2048];
extern unsigned short symbian_palette16[256];
void launch_emulatorL(void* aEmulatorPointer);
void UpdateScreenData();
char buffer[2048];

void usleep(int aValue)
{
	User::AfterHighRes(aValue);
}

int	symbian_printf		(const char * text, ...)
{
#ifdef __WINS__
	va_list args;
	va_start(args, text);
	vsprintf(buffer, text, args);
	va_end(args);
	RDebug::Printf(buffer);
#endif
	return 0;
}

int	 SymbianScandir(const char * aPath, struct SymbianDirent *** aDirEntList,
	    int (*)(struct SymbianDirent *), int (*)(const void *, const void *))
{
	TInt cnt = 0;
	TFileName path;
	TPtrC8 ptr((const unsigned char*) (aPath), strlen(aPath));
	path.Copy(ptr);
	
	if(path.Length() == 0)
	{		
		TDriveList driveList;
		TFileName driveLetter;	
		TInt countedDrives=0;
		CEikonEnv::Static()->FsSession().DriveList(driveList);

		for(TInt drive=0;drive<driveList.Length();drive++)
		{
			if(driveList[drive]!=0)
			{
				countedDrives++;
			}			
		}	
			
		*aDirEntList = (struct SymbianDirent **) malloc(countedDrives*sizeof(struct SymbianDirent**));
		countedDrives=0;

		for(TInt drive=0;drive<driveList.Length();drive++)
		{
			if(driveList[drive]!=0)
			{
				TBuf8<16> form;
				form.Format(_L8("%c:\\"),drive+65);
				form.ZeroTerminate();
				(*aDirEntList)[countedDrives] =(struct SymbianDirent*)  malloc(sizeof(struct SymbianDirent));
				strcpy((*aDirEntList)[countedDrives]->d_name, (char*) form.Ptr());
				(*aDirEntList)[countedDrives]->d_namlen = form.Length();
				(*aDirEntList)[countedDrives]->d_fileno = countedDrives;
				(*aDirEntList)[countedDrives]->d_reclen = 0;
				(*aDirEntList)[countedDrives]->d_type = DT_DIR;
				countedDrives++;				
			}			
		}	
		cnt = countedDrives;
	}
	else
	{
		CDir* entryList = NULL;
		if(path[path.Length()-1] != '\\' && path[path.Length()-1] != '/')
		{
			path.Append(_L("\\"));
		}

		TInt error  = CEikonEnv::Static()->FsSession().GetDir(path, 0xFFFF, ESortByName, entryList);
		if(error == KErrNone)
		{
			cnt = entryList->Count()+1;
			*aDirEntList = (struct SymbianDirent **) malloc(cnt*sizeof(struct SymbianDirent**));
		
			TBuf8<256> fname;
			for(TInt loop = 0;loop<cnt;loop++)
			{
				(*aDirEntList)[loop] =(struct SymbianDirent*)  malloc(sizeof(struct SymbianDirent));
				(*aDirEntList)[loop]->d_fileno = loop;

				if(loop == 0)
				{
					strcpy((*aDirEntList)[loop]->d_name, "..");
					(*aDirEntList)[loop]->d_type = DT_DIR;
					(*aDirEntList)[loop]->d_namlen = 2;
					(*aDirEntList)[loop]->d_reclen = 0;
				}
				else
				{					
					fname.Copy((*entryList)[loop-1].iName);
					fname.PtrZ();
					strcpy((*aDirEntList)[loop]->d_name, (char*) fname.Ptr());
					(*aDirEntList)[loop]->d_namlen = fname.Length();					
					(*aDirEntList)[loop]->d_reclen = (*entryList)[loop-1].iSize;
					(*aDirEntList)[loop]->d_type = (*entryList)[loop-1].IsDir()?DT_DIR:DT_REG;
				}
			}
		}
		else
		{
			cnt = error;
		}

		delete entryList;
	}
	return cnt;
}



void SetDefaults(void)
{
	memset(&Settings,0,sizeof(Settings));
	Settings.frameskip = -1; // auto
	Settings.sound_rate = 22050;	
	Settings.turbo_rate_add = (8*2 << 24) / 60 + 1; // 8Hz turbofire
	Settings.gamma = 100;
	Settings.sstate_confirm = 1;
	Settings.stretch = 0;
	Settings.orientation = 0;
	Settings.networktype = 0;
	// default controls, RLDU SEBA

	Settings.ScanCodes[ENesUpKey1] = EStdKeyUpArrow;
	Settings.ScanCodes[ENesDownKey1] = EStdKeyDownArrow; 
	Settings.ScanCodes[ENesLeftKey1] = EStdKeyLeftArrow; 
	Settings.ScanCodes[ENesRightKey1] = EStdKeyRightArrow; 
	Settings.ScanCodes[ENesAKey1] = EStdKeyDevice1 ; 
	Settings.ScanCodes[ENesBKey1] = EStdKeyDevice0; 
	Settings.ScanCodes[ETurboAKey1] ='4'; 
	Settings.ScanCodes[ETurboBKey1] = '5'; 
	Settings.ScanCodes[ENesModeKey1] = '0';
	Settings.ScanCodes[ENesStartKey1] = EStdKeyDevice3;

	Settings.ScanCodes[ENesUpKey2] = 0;
	Settings.ScanCodes[ENesDownKey2] = 0; 
	Settings.ScanCodes[ENesLeftKey2] = 0; 
	Settings.ScanCodes[ENesRightKey2] = 0; 
	Settings.ScanCodes[ENesAKey2] = 0; 
	Settings.ScanCodes[ENesBKey2] = 0; 
	Settings.ScanCodes[ETurboAKey2] = 0; 
	Settings.ScanCodes[ETurboBKey2] = 0; 
	Settings.ScanCodes[ENesModeKey2] = 0;
	Settings.ScanCodes[ENesStartKey2] = 0;

	Settings.ScanCodes[ENesSaveState] = 0;
	Settings.ScanCodes[ENesLoadState] = 0;
	Settings.ScanCodes[ENesNextState] = 0;
	Settings.ScanCodes[ENesPrevState] = 0;
	Settings.ScanCodes[ENesInsertEject] = 0;
	Settings.ScanCodes[ENesSelectDisk] = 0;
	Settings.ScanCodes[ENesInsertCoin] = 0;
	/*Settings.ScanCodes[ENesULKey] = 0;
	Settings.ScanCodes[ENesURKey] = 0;
	Settings.ScanCodes[ENesDRKey] = 0; 
	Settings.ScanCodes[ENesDLKey] = 0;*/
	scan_name[EStdKeyNull] = "";
	scan_name[EStdKeyBackspace] = "BkSpc";
	scan_name[EStdKeyTab] = "Tab";
	scan_name[EStdKeyEnter] = "Enter";
	scan_name[EStdKeyEscape] = "Esc";
	scan_name[EStdKeySpace] = "Spc";
	scan_name[EStdKeyPrintScreen] = "PrntScr";
	scan_name[EStdKeyPause] = "Pause";
	scan_name[EStdKeyHome] = "Home";
	scan_name[EStdKeyEnd] = "End";
	scan_name[EStdKeyPageUp] = "PgUp";
	scan_name[EStdKeyPageDown] = "PgDwn";
	scan_name[EStdKeyInsert] = "Ins";
	scan_name[EStdKeyDelete] = "Del";
	scan_name[EStdKeyLeftArrow] = "Left";
	scan_name[EStdKeyRightArrow] = "Right";
	scan_name[EStdKeyUpArrow] = "Up";
	scan_name[EStdKeyDownArrow] = "Down";
	scan_name[EStdKeyLeftShift] = "LShft";
	scan_name[EStdKeyRightShift] = "RShft";
	scan_name[EStdKeyLeftAlt] = "LAlt";
	scan_name[EStdKeyRightAlt] = "RAlt";
	scan_name[EStdKeyLeftCtrl] = "LCtrl";
	scan_name[EStdKeyRightCtrl] = "RCtrl";
	scan_name[EStdKeyLeftFunc] = "LFunc";
	scan_name[EStdKeyRightFunc] = "RFunc";
	scan_name[EStdKeyCapsLock] = "Caps";
	scan_name[EStdKeyNumLock] = "NumLck";
	scan_name[EStdKeyScrollLock] = "ScrLck";
	scan_name[EStdKeyF1] = "F1";
	scan_name[EStdKeyF2] = "F2";
	scan_name[EStdKeyF3] = "F3";
	scan_name[EStdKeyF4] = "F4";
	scan_name[EStdKeyF5] = "F5";
	scan_name[EStdKeyF6] = "F6";
	scan_name[EStdKeyF7] = "F7";
	scan_name[EStdKeyF8] = "F8";
	scan_name[EStdKeyF9] = "F9";
	scan_name[EStdKeyF10] = "F10";
	scan_name[EStdKeyF11] = "F11";
	scan_name[EStdKeyF12] = "F12";
	scan_name[EStdKeyF13] = "F13";
	scan_name[EStdKeyF14] = "F14";
	scan_name[EStdKeyF15] = "F15";
	scan_name[EStdKeyF16] = "F16";
	scan_name[EStdKeyF17] = "F17";
	scan_name[EStdKeyF18] = "F18";
	scan_name[EStdKeyF19] = "F19";
	scan_name[EStdKeyF20] = "F20";
	scan_name[EStdKeyF21] = "F21";
	scan_name[EStdKeyF22] = "F22";
	scan_name[EStdKeyF23] = "F23";
	scan_name[EStdKeyF24] = "F24";
	scan_name[EStdKeyXXX] = "F25";
	scan_name[EStdKeyComma] = ",";
	scan_name[EStdKeyFullStop] = ".";
	scan_name[EStdKeyForwardSlash] = "/";
	scan_name[EStdKeyBackSlash] = "\\";
	scan_name[EStdKeySemiColon] = ";";
	scan_name[EStdKeySingleQuote] = "'";
	scan_name[EStdKeyHash] = "#";
	scan_name[EStdKeySquareBracketLeft] = "[";
	scan_name[EStdKeySquareBracketRight] = "]";
	scan_name[EStdKeyMinus] = "-";
	scan_name[EStdKeyEquals] = "=";
	scan_name[EStdKeyNkpForwardSlash] = "/";
	scan_name[EStdKeyNkpAsterisk] = "*";
	scan_name[EStdKeyNkpMinus] = "-";
	scan_name[EStdKeyNkpPlus] = "+";
	scan_name[EStdKeyNkpEnter] = "Enter";
	scan_name[EStdKeyNkp1] = "1";
	scan_name[EStdKeyNkp2] = "2";
	scan_name[EStdKeyNkp3] = "3";
	scan_name[EStdKeyNkp4] = "4";
	scan_name[EStdKeyNkp5] = "5";
	scan_name[EStdKeyNkp6] = "6";
	scan_name[EStdKeyNkp7] = "7";
	scan_name[EStdKeyNkp8] = "8";
	scan_name[EStdKeyNkp9] = "9";
	scan_name[EStdKeyNkp0] = "0";
	scan_name[EStdKeyNkpFullStop] = ".";
	scan_name[EStdKeyMenu] = "Menu";
	scan_name[EStdKeyBacklightOn] = "BOn";
	scan_name[EStdKeyBacklightOff] = "BOff";
	scan_name[EStdKeyBacklightToggle] = "BTog";
	scan_name[EStdKeyIncContrast] = "IncC";
	scan_name[EStdKeyDecContrast] = "DecC";
	scan_name[EStdKeySliderDown] = "SDwn";
	scan_name[EStdKeySliderUp] = "SUp";
	scan_name[EStdKeyDictaphonePlay] = "DPlay";
	scan_name[EStdKeyDictaphoneStop] = "DStop";
	scan_name[EStdKeyDictaphoneRecord] = "DRec";
	scan_name[EStdKeyHelp] = "Help";
	scan_name[EStdKeyOff] = "Off";
	scan_name[EStdKeyDial] = "Dial";
	scan_name[EStdKeyIncVolume] = "VInc";
	scan_name[EStdKeyDecVolume] = "VDec";
	scan_name[EStdKeyDevice0] = "Dev0";
	scan_name[EStdKeyDevice1] = "Dev1";
	scan_name[EStdKeyDevice2] = "Dev2";
	scan_name[EStdKeyDevice3] = "Dev3";
	scan_name[EStdKeyDevice4] = "Dev4";
	scan_name[EStdKeyDevice5] = "Dev5";
	scan_name[EStdKeyDevice6] = "Dev6";
	scan_name[EStdKeyDevice7] = "Dev7";
	scan_name[EStdKeyDevice8] = "Dev8";
	scan_name[EStdKeyDevice9] = "Dev9";
	scan_name[EStdKeyDeviceA] = "DevA";
	scan_name[EStdKeyDeviceB] = "DevB";
	scan_name[EStdKeyDeviceC] = "DevC";
	scan_name[EStdKeyDeviceD] = "DevD";
	scan_name[EStdKeyDeviceE] = "DevE";
	scan_name[EStdKeyDeviceF] = "DevF";
	scan_name[EStdKeyApplication0] = "App0";
	scan_name[EStdKeyApplication1] = "App1";
	scan_name[EStdKeyApplication2] = "App2";
	scan_name[EStdKeyApplication3] = "App3";
	scan_name[EStdKeyApplication4] = "App4";
	scan_name[EStdKeyApplication5] = "App5";
	scan_name[EStdKeyApplication6] = "App6";
	scan_name[EStdKeyApplication7] = "App7";
	scan_name[EStdKeyApplication8] = "App8";
	scan_name[EStdKeyApplication9] = "App9";
	scan_name[EStdKeyApplicationA] = "AppA";
	scan_name[EStdKeyApplicationB] = "AppB";
	scan_name[EStdKeyApplicationC] = "AppC";
	scan_name[EStdKeyApplicationD] = "AppD";
	scan_name[EStdKeyApplicationE] = "AppE";
	scan_name[EStdKeyApplicationF] = "AppF";
	scan_name[EStdKeyYes] = "Yes";
	scan_name[EStdKeyNo] = "No";
	scan_name[EStdKeyIncBrightness] = "BInc";
	scan_name[EStdKeyDecBrightness] = "BDec";
	scan_name[EStdKeyKeyboardExtend] = "KExt";
	scan_name[EStdKeyDevice10] = "Dev10";
	scan_name[EStdKeyDevice11] = "Dev11";
	scan_name[EStdKeyDevice12] = "Dev12";
	scan_name[EStdKeyDevice13] = "Dev13";
	scan_name[EStdKeyDevice14] = "Dev14";
	scan_name[EStdKeyDevice15] = "Dev15";
	scan_name[EStdKeyDevice16] = "Dev16";
	scan_name[EStdKeyDevice17] = "Dev17";
	scan_name[EStdKeyDevice18] = "Dev18";
	scan_name[EStdKeyDevice19] = "Dev19";
	scan_name[EStdKeyDevice1A] = "Dev1A";
	scan_name[EStdKeyDevice1B] = "Dev1B";
	scan_name[EStdKeyDevice1C] = "Dev1C";
	scan_name[EStdKeyDevice1D] = "Dev1D";
	scan_name[EStdKeyDevice1E] = "Dev1E";
	scan_name[EStdKeyDevice1F] = "Dev1F";
	scan_name[EStdKeyApplication10] = "App10";
	scan_name[EStdKeyApplication11] = "App11";
	scan_name[EStdKeyApplication12] = "App12";
	scan_name[EStdKeyApplication13] = "App13";
	scan_name[EStdKeyApplication14] = "App14";
	scan_name[EStdKeyApplication15] = "App15";
	scan_name[EStdKeyApplication16] = "App16";
	scan_name[EStdKeyApplication17] = "App17";
	scan_name[EStdKeyApplication18] = "App18";
	scan_name[EStdKeyApplication19] = "App19";
	scan_name[EStdKeyApplication1A] = "App1A";
	scan_name[EStdKeyApplication1B] = "App1B";
	scan_name[EStdKeyApplication1C] = "App1C";
	scan_name[EStdKeyApplication1D] = "App1D";
	scan_name[EStdKeyApplication1E] = "App1E";
	scan_name[EStdKeyApplication1F] = "App1F";
}

void Update_input(void* aEmulatorPointer)
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->UpdateInputL();
 }
 
 void SelectRomL(void* aEmulatorPointer)
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->SelectRomL();
 }

 void WriteSoundL(void* aEmulatorPointer, short int*  aBuffer, int aCount)
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->WriteSoundL(aBuffer, aCount);
 }

 void UpdateSampleRateL(void* aEmulatorPointer)
 {
 	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->UpdateSampleRateL();
 }

 void ClearScreen(void* aEmulatorPointer)
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->ClearScreen();
 }

 void UpdateScreenL(void* aEmulatorPointer)
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->UpdateScreenL();
 }

 void GetKeyStatusL(void* aEmulatorPointer, short unsigned int* aPlayer1, short unsigned int* aPlayer2)	 
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->GetKeyStatusL(*aPlayer1, *aPlayer2);
 }


 void GetMouseStatusL(void* aEmulatorPointer, unsigned  char* aX, unsigned  char* aY)	 
 {
	 static_cast<CGpfceEmulator*>(aEmulatorPointer)->GetMouseStatusL(*aX, *aY);
 }


 //char *netplayhost=0;
 //int Port=0xFCE;
 
 int FCEUD_NetworkConnect(void)
 {
#ifdef NETWORK

	 return static_cast<CGpfceEmulator*>(emulator_pointer)->NetworkConnectL();
#else
 	 return(1);  
#endif
 }
 
 /* 0 on failure, 1 on success, -1 if it would block and blocking is not
 specified.
 */
 
 int FCEUD_NetworkRecvData(uint8 *data, uint32 len, int block)
 {
#ifdef NETWORK
	 return static_cast<CGpfceEmulator*>(emulator_pointer)->NetworkRecvDataL(data, len, block);
#else  
	 return 1;
#endif
 }
 
 /* 0 on failure, 1 on success.  This function should always block. */
 
 int FCEUD_NetworkSendData(uint8 *Value, uint32 len)
 {
#ifdef NETWORK
 	 return static_cast<CGpfceEmulator*>(emulator_pointer)->NetworkSendDataL(Value, len);
#else
	 return 0;
#endif
 }
 
 void FCEUD_NetworkClose(void)
 {
#ifdef NETWORK
	 static_cast<CGpfceEmulator*>(emulator_pointer)->NetworkCloseL();
	 /*if(Socket>0)
		 close(Socket);
	 Socket=-1;*/
#endif
 }
 


}

CGpfceDoc::~CGpfceDoc()
{
	CloseSTDLIB();
}

CEikAppUi* CGpfceDoc::CreateAppUiL()
{
	CGpfceAppUi* appUi = new (ELeave) CGpfceAppUi;
	return appUi;
}

CGpfceAppUi::CGpfceAppUi()
{

}

CGpfceAppUi::~CGpfceAppUi()
{
#ifdef UIQ3
#else
	delete iCoreTarget;
	delete iInterfaceSelector;
	RemoveFromStack(iEmu);
	delete iEmu;
#endif

}

void CGpfceAppUi::ConstructL()
{
#ifdef UIQ3
	BaseConstructL();
#else
	CAknAppUi::BaseConstructL(CAknAppUi::EAknEnableSkin);
    iInterfaceSelector = CRemConInterfaceSelector::NewL();
    iCoreTarget = CRemConCoreApiTarget::NewL(*iInterfaceSelector, *this);
    TRAPD(err, iInterfaceSelector->OpenTargetL());

#endif
	iEmu = CGpfceEmulator::NewL(*this);
#ifdef UIQ3
	AddViewL(*iEmu);
#else
	AddToStackL(iEmu);
	SetFullScreenApp(ETrue);
	SetKeyBlockMode(ENoKeyBlock);
#endif
}

void CGpfceAppUi::HandleCommandL(TInt aCommand)
{
	if(aCommand == EEikCmdExit)
	{
		Exit();
	}
}

void CGpfceAppUi::GetKeyStatusL(TUint16& aPlayer1, TUint16& aPlayer2)
{
	aPlayer1 = iPad1;
	aPlayer2 = iPad2;
}

extern "C" int last_key;

const TUint16 KLandscapeModeKeys[4] = {0x80,0x40,0x10,0x20};
const TUint16 KLandscape2ModeKeys[4] = {0x40,0x80,0x20,0x10};

#ifndef UIQ3

/** 
 A command has been received. 
 @param aOperationId The operation ID of the command.
 @param aButtonAct The button action associated with the command.
 */
void CGpfceAppUi::MrccatoCommand(TRemConCoreApiOperationId aOperationId, 
								TRemConCoreApiButtonAction aButtonAct)
{
	if(running_emu)
	{
		if(aButtonAct == ERemConCoreApiButtonPress || aButtonAct == ERemConCoreApiButtonRelease)
		{
			TInt scanCode = aOperationId<<16;
			TUint16 bitVal=0;
			
			for(TInt loop=0;loop<ENesNoKeys;loop++)
			{
				if(scanCode == Settings.ScanCodes[loop])
				{
					
					switch(loop)
					{
					case ENesUpKey1:
					case ENesDownKey1:
					case ENesLeftKey1:
					case ENesRightKey1:
						
					case ENesUpKey2:
					case ENesDownKey2:
					case ENesLeftKey2:
					case ENesRightKey2:
						{
							TInt offset = 0;
							switch(Settings.orientation)
							{
							case 0:
								bitVal=KBitValTable[loop];
								break;
							case 1:
								if(loop>=ENesUpKey2)
								{
									offset = ENesUpKey2;
								}
								bitVal= KLandscapeModeKeys[loop-offset];
								
								break;
							case 2:
								if(loop>=ENesUpKey2)
								{
									offset = ENesUpKey2;
								}
								bitVal= KLandscape2ModeKeys[loop-offset];
								break;
							}
						}
						break;
					default:
						bitVal=KBitValTable[loop];
						break;
					}
					
					if(aButtonAct == ERemConCoreApiButtonRelease)
					{
						if(loop<ENesUpKey2 || loop > ENesModeKey2)
						{
							iPad1=iPad1&(65535-bitVal); // remove bit
						}
						else
						{
							iPad2=iPad2&(65535-bitVal); // remove bit
						}
					}
					else 
					{
						if(loop<ENesUpKey2 || loop > ENesModeKey2)
						{
							iPad1=(iPad1|bitVal); // set bit
						}
						else
						{
							iPad2=(iPad2|bitVal); // set bit
						}
					}	
					
					break;
				}
			}									
		}
	}
	else if(aButtonAct == ERemConCoreApiButtonPress && setting_key)
	{
		setting_key_scancode = aOperationId<<16;
		setting_key = 0;
	}	
		
}
#endif


TKeyResponse CGpfceAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
	
	if(running_emu && !waiting_for_input)
	{
		if(aType == EEventKeyDown || aType == EEventKeyUp)
		{
		
#ifdef UIQ3
		TInt scanCode = RemapScanCode(aKeyEvent.iScanCode);
#else
		TInt scanCode = aKeyEvent.iScanCode;
#endif				
			TUint16 bitVal=0;
		
			for(TInt loop=0;loop<ENesNoKeys;loop++)
			{
				if(scanCode == Settings.ScanCodes[loop])
				{

					switch(loop)
					{
					case ENesUpKey1:
					case ENesDownKey1:
					case ENesLeftKey1:
					case ENesRightKey1:
						
					case ENesUpKey2:
					case ENesDownKey2:
					case ENesLeftKey2:
					case ENesRightKey2:
						{
							TInt offset = 0;
							switch(Settings.orientation)
							{
							case 0:
								bitVal=KBitValTable[loop];
								break;
							case 1:
								if(loop>=ENesUpKey2)
								{
									offset = ENesUpKey2;
								}
								bitVal= KLandscapeModeKeys[loop-offset];
								
								break;
							case 2:
								if(loop>=ENesUpKey2)
								{
									offset = ENesUpKey2;
								}
								bitVal= KLandscape2ModeKeys[loop-offset];
								break;
							}
						}
						break;
					default:
						bitVal=KBitValTable[loop];
						break;
					}
					
					if(aType == EEventKeyUp)
					{
						if(loop<ENesUpKey2 || loop > ENesModeKey2)
						{
							iPad1=iPad1&(65535-bitVal); // remove bit
						}
						else
						{
							iPad2=iPad2&(65535-bitVal); // remove bit
						}
					}
					else 
					{
						if(loop<ENesUpKey2 || loop > ENesModeKey2)
						{
							iPad1=(iPad1|bitVal); // set bit
						}
						else
						{
							iPad2=(iPad2|bitVal); // set bit
						}
					}	
					
					break;
				}
			}
								
						
		}

		if(aType == EEventKey && (aKeyEvent.iCode == EKeyBackspace || aKeyEvent.iCode == EKeyDelete || aKeyEvent.iCode == EKeyEscape))
		{
			FCEUI_CloseGame();
		}
	}
	else // not running _emu
	{
		// reset input for pad1
		if(waiting_for_input)
		{
			iPad1 = 0;
		}

		if(!setting_key && aType == EEventKey)
		{
			switch(aKeyEvent.iCode)
			{
#ifdef UIQ3
			case EDeviceKeyFourWayLeft:
			case '4':
#endif
			case EKeyLeftArrow:
				last_key|=SYMBIAN_LEFT;
				break;
#ifdef UIQ3
			case EDeviceKeyFourWayRight:
			case '6':
#endif
			case EKeyRightArrow:
				last_key|=SYMBIAN_RIGHT;
				break;
#ifdef UIQ3
			case EDeviceKeyFourWayUp:
			case EDeviceKeyTwoWayUp:
			case '2':
#endif
			case EKeyUpArrow:
				last_key|=SYMBIAN_UP;
				break;
#ifdef UIQ3
			case EDeviceKeyFourWayDown:
			case EDeviceKeyTwoWayDown:
			case '8':
#endif
			case EKeyDownArrow:
				last_key|=SYMBIAN_DOWN;
				break;
#ifdef UIQ3
			case EDeviceKeyDone:
			case '5':
//			case EDeviceKeyCancel:			

#else
			case EKeyDevice1 :
#endif				
				last_key|=(SYMBIAN_SELECT4);
				break;
			case EKeyBackspace:
			case EKeyDelete:
			case EKeyEscape:
				last_key|=(SYMBIAN_SELECT2);
				break;
			
			case EKeyEnter:
#ifdef UIQ3
			case EDeviceKeyAction:
			case EDeviceKeySoftkey1:			
#else
			case EKeyDevice3:
			case EKeyDevice0: 
#endif
				last_key|=(SYMBIAN_SELECT3);
				break;
				
			}
		}
		else if(aType == EEventKeyDown && setting_key)
		{
#ifdef UIQ3
			setting_key_scancode = RemapScanCode(aKeyEvent.iScanCode);
#else
			setting_key_scancode = aKeyEvent.iScanCode;
#endif
			setting_key = 0;
		}
	}
	return EKeyWasConsumed;
}

void CGpfceAppUi::HandleScreenDeviceChangedL()
{
#ifdef UIQ3
#else
	CAknAppUi::HandleScreenDeviceChangedL();
#endif
	if(iEmu != NULL)
		{
			iEmu->UpdateScreenParamsL();
		}

	if(running_emu)
	{
		FCEUI_CloseGame();
	}

}

	
void CGpfceAppUi::HandleForegroundEventL(TBool aForeground)
{
	if(iEmu != NULL)
	{
		iEmu->iForeground=aForeground;
	}

	if(!aForeground)
	{
		if(iEmu != NULL)
		{
			iEmu->AbortNow(RDirectScreenAccess::ETerminateCancel);
		}

		FCEUI_CloseGame();
	}
	else
	{
		if(iEmu != NULL)
		{
		iEmu->Restart(RDirectScreenAccess::ETerminateCancel);
		iEmu->UpdateScreenL();
		}	

#ifdef UIQ3
#else
		SetKeyBlockMode(ENoKeyBlock);
#endif
	}
	
}



CGpfceEmulator* CGpfceEmulator::NewL(CGpfceAppUi& aAppUi)
{
	CGpfceEmulator* self = new (ELeave) CGpfceEmulator(aAppUi);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CGpfceEmulator::~CGpfceEmulator()
{
	iDsa->Cancel();
	delete iDsa;
	delete iAsynchStart;
	delete iIdleCallBack;
	delete iSndStream;
	delete iBackBuffer;
	iSocket.Close();
	iSocketServ.Close();
}

CGpfceEmulator::CGpfceEmulator(CGpfceAppUi& aAppUi):
#ifdef UIQ3
CQikViewBase(aAppUi, KNullViewId),
#endif
iAppUi(aAppUi)
{
}

void CGpfceEmulator::UpdateScreenParamsL()
{
	if(iBackBuffer == NULL || (iBackBuffer != NULL && iEikEnv->ScreenDevice()->SizeInPixels() != iBackBuffer->SizeInPixels()))
	{
		iDrawingOn = EFalse;	

		delete iBackBuffer;
		iBackBuffer = NULL;
		iBackBuffer= new (ELeave)CFbsBitmap;
		iBackBuffer->Create(iEikEnv->ScreenDevice()->SizeInPixels(),EColor64K);
		TBitmapUtil util(iBackBuffer);
		util.Begin(TPoint(0,0));

		TSize sz=iBackBuffer->SizeInPixels();
	
		TInt dataSize=sz.iWidth*sz.iHeight*2;
	
		TPtr8 ptr(reinterpret_cast<unsigned char*>(iBackBuffer->DataAddress()),dataSize,dataSize);
		ptr.Fill(0);
		util.End();

		SetRect(TRect(TPoint(0,0), iEikEnv->ScreenDevice()->SizeInPixels()));
		delete iDsa;
		iDsa = NULL;
		iDsa=CDirectScreenAccess::NewL(iEikEnv->WsSession(), *iEikEnv->ScreenDevice(), 
									Window(), *this);
		iDsa->StartL();
		TRegionFix<1> region(iBackBuffer->SizeInPixels());
		
		iDsa->Gc()->SetClippingRegion(region);
	
		iDrawingOn=ETrue;	
		iDsa->Gc()->BitBlt(TPoint(0,0), iBackBuffer, iBackBuffer->SizeInPixels());
		iDsa->ScreenDevice()->Update();
		gScreenSize = iBackBuffer->SizeInPixels();
		UpdateScreenData();
	}
}

void CGpfceEmulator::ConstructL()
{
	iLastPoint = TPoint(255, 255);
#ifdef UIQ3
	TQikViewMode viewMode;
	viewMode.SetFullscreen();
	BaseConstructL();
#endif
	CreateWindowL();
	ActivateL();
	SetFocus(ETrue);
#ifdef UIQ3
	PreemptViewConstructionL();
	SetViewModeL(viewMode);
#endif
	iIdleCallBack = new (ELeave) CAsyncCallBack( TCallBack(IdleCallBackStop,this), CActive::EPriorityIdle);
	iEikEnv = CEikonEnv::Static();

	UpdateScreenParamsL();
	
	iSndStream = CMdaAudioOutputStream::NewL(*this);
	iAudioSettings.Query();
	iAudioSettings.iSampleRate = TMdaAudioDataSettings::ESampleRate22050Hz;
	iAudioSettings.iChannels = TMdaAudioDataSettings::EChannelsMono;
	iAudioSettings.iFlags = 0;
	iAudioSettings.iVolume = iAudioSettings.iMaxVolume/2;
	iSndStream->Open(&iAudioSettings);
	CActiveScheduler::Start(); // wait for open
	
	iAsynchStart = new (ELeave) CAsyncCallBack(TCallBack(AsynchStartupL, this), CActive::EPriorityStandard);
	iAsynchStart->CallBack();
	MakeVisible(ETrue);
	iSocketServ.Connect();
	EnableDragEvents();
}

void CGpfceEmulator::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	if(iBackBuffer != NULL)
	{
		gc.BitBlt(TPoint(0,0), iBackBuffer);
	}
}

void CGpfceEmulator::StartEmulatorL()
{
	lastLoadedGameName[0] = 0;


	TRAP_IGNORE(launch_emulatorL(this));
	iAppUi.Exit();
}


void CGpfceEmulator::SelectRomL()
{
	TFileName romName;
#ifdef UIQ3
#else
	if(AknCommonDialogs::RunSelectDlgLD(romName, 0, _L("Select ROM!")))
#endif
	{
		TPtr8 ptr((TUint8*) lastLoadedGameName,2048);
		ptr.Copy(romName);
		ptr.ZeroTerminate();
	}
}
TInt CGpfceEmulator::AsynchStartupL(TAny* aEmuClass)
{
	static_cast<CGpfceEmulator*>(aEmuClass)->StartEmulatorL();
	return KErrNone;
}

TInt CGpfceEmulator::IdleCallBackStop(TAny* /*aAppUi*/)
{
	CActiveScheduler::Stop();
	return 0;
}

void CGpfceEmulator::UpdateSampleRateL()
{
	iNewSampleRate = TMdaAudioDataSettings::ESampleRate22050Hz;
	switch(Settings.sound_rate)
	{
	case 8000:
		iNewSampleRate = TMdaAudioDataSettings::ESampleRate8000Hz;
		break;
	case 11025:
		iNewSampleRate = TMdaAudioDataSettings::ESampleRate11025Hz;
		break;
	case 16000:
		iNewSampleRate = TMdaAudioDataSettings::ESampleRate16000Hz;
		break;
	case 22050:
		iNewSampleRate = TMdaAudioDataSettings::ESampleRate22050Hz;
		break;
	case 44100:
		iNewSampleRate = TMdaAudioDataSettings::ESampleRate44100Hz;
		break;
	}

	if(iNewSampleRate != 0)
	{		
		iAudioBuffer[0].Zero();
		iAudioBuffer[1].Zero();	
		iBufferCnt = 0;
		iCurrentBuffer = 0;
		
		TRAPD(err,iSndStream->SetAudioPropertiesL(iNewSampleRate, TMdaAudioDataSettings::EChannelsMono));
		iSndStream->SetVolume((iSndStream->MaxVolume()*soundvol)/100);	
		iNewSampleRate = 0;
	}
}

void CGpfceEmulator::MaoscOpenComplete(TInt aError)
{
	if(aError == KErrNone)
	{
		iSndStream->SetPriority(EPriorityMuchMore, EMdaPriorityPreferenceNone);
		iSndStream->SetVolume((iSndStream->MaxVolume()*soundvol)/100);		
	}

	CActiveScheduler::Stop();
}

void CGpfceEmulator::MaoscBufferCopied(TInt /*aError*/, const TDesC8& /*aBuffer*/)
{
}

void CGpfceEmulator::MaoscPlayComplete(TInt aError)
{
	if(aError != KErrNone)
	{
		iSndStream->SetVolume((iSndStream->MaxVolume()*soundvol)/100);	
	}

	iBufferCnt = 0;
	iAudioBuffer[0].Zero();
	iAudioBuffer[1].Zero();
}

void CGpfceEmulator::WriteSoundL(TInt16* aBuffer, TInt aCount)
{
	TPtrC8 ptr((TUint8*) aBuffer, aCount*2);
	iAudioBuffer[iCurrentBuffer].Append(ptr);
	iBufferCnt++;
	if(iBufferCnt == 5)
	{
		iSndStream->WriteL(iAudioBuffer[iCurrentBuffer]);
		iBufferCnt = 0;
		iCurrentBuffer = 1-iCurrentBuffer;
		iAudioBuffer[iCurrentBuffer].Zero();
	
	}
}

TInt CGpfceEmulator::NetworkConnectL()
{
	TInt err = 0;
	if(Settings.networktype == 0) // bluetooth
	{
		iSocket.Close();
		TInt err = iSocket.Open( iSocketServ, KL2CAPDesC );
	}
	else
	{
		iSocket.Close();
		iListeningSocket.Close();
		iIpAddress.SetPort(Port);
		iIpAddress.SetFamily(KAfInet);
	
		
		//struct sockaddr_in sockn;
		//int TSocket;
		
		// memset(&sockn,0,sizeof(sockn));
		//sockn.sin_family=AF_INET;
		// sockn.sin_port=htons(Port);
		
		/*	if((TSocket=socket(AF_INET, SOCK_STREAM, 0))<0)
		{
		puts("Error creating socket.");
		return(0);
	}*/
		
		if(FSettings.NetworkPlay/*netplay*/==1)		// Be a server. 
		{
			//sockn.sin_addr.s_addr=INADDR_ANY;
			TInt err = iListeningSocket.Open(iSocketServ, KAfInet,KSockStream, KProtocolInetTcp);
			if(err != KErrNone)
			{			
				FCEU_printf("Error creating socket.");
				return 0;
			}
			
			err = iSocket.Open( iSocketServ );

			if(err != KErrNone)
			{			
				FCEU_printf("Error creating socket2.");
				return 0;
			}
			
			iIpAddress.SetAddress(KInetAddrAny);

			if(iListeningSocket.Bind(iIpAddress) != KErrNone)
			{
				FCEU_printf("Error binding to socket.");
				iListeningSocket.Close();
				iSocket.Close();
				return 0;
			}

			/*if(bind(TSocket, (struct sockaddr *)&sockn, sizeof(sockn))<0)
			{
				close(TSocket);
				puts("Error binding to socket."); 
				return(0);
			}
			*/
			if(iListeningSocket.Listen(1) != KErrNone)
			{
				FCEU_printf("Error listening on socket.");
				iListeningSocket.Close();
				iSocket.Close();
				return 0;
			}
	
			CActiveWaiter* waiter = new (ELeave) CActiveWaiter(err);

			iListeningSocket.Accept(iSocket, waiter->iStatus);
			waiter->Start();
			
			if(err != KErrNone)
			{
				iListeningSocket.Close();
				iSocket.Close();
				return 0;
			}
			iListeningSocket.Close();
			//iSocket.SetOpt(opname, KSolInetTcp, option);		
/*			if(listen(TSocket, 1)<0)
			{
				puts("Error listening on socket.");
				close(TSocket);
				return(0);
			}*/
			/*{
				socklen_t len=sizeof(sockn);
				
				printf("Accepting connection on port %d...\n",Port);
				if((Socket=accept(TSocket,(struct sockaddr *)&sockn,&len))<0 )
				{
					puts("Error accepting a connection.");
					close(TSocket);
					return(0);
				}
				close(TSocket);
			  }*/
			
		}
		else // Connect as a client if not a server. 
		{
			err = iSocket.Open(iSocketServ, KAfInet,KSockStream, KProtocolInetTcp);
			if(err != KErrNone)
			{			
				FCEU_printf("Error creating socket.");
				return 0;
			}		
			struct hostent *Host;

			TPtrC8 ptr((unsigned char*) netplayhost, strlen(netplayhost));
			TBuf<20> ipAddrBuf;
			ipAddrBuf.Copy(ptr);
			err = iIpAddress.Input(ipAddrBuf);
			if(err != KErrNone) // Do host resolve
			{
				FCEU_printf("Error getting network host entry.");
			}
			/*if((sockn.sin_addr.s_addr=inet_addr(netplayhost))==INADDR_NONE)
			{
				if(!(Host=gethostbyname(netplayhost)))
				{
					puts("Error getting network host entry.");
					return(0);
				}
				memcpy(&sockn.sin_addr,Host->h_addr,Host->h_length);
			}  */
			TInt available;
			iSocket.GetOpt(KSIPeekData, KSOLSocket, available);
			FCEU_printf("Attempting to connect to %s...\n",netplayhost);
			CActiveWaiter* waiter = new (ELeave) CActiveWaiter(err);
			iSocket.Connect(iIpAddress, waiter->iStatus);
			waiter->Start();

			if(err != KErrNone)
			{
				FCEU_printf("Error connecting to remote host.");
				return 0;
			}
			
/*			if( connect(TSocket, (struct sockaddr *)&sockn, sizeof(sockn)) <0 )
			{
				puts("Error connecting to remote host.");
				close(TSocket);
				return(0);
			}
			Socket=TSocket;*/
		}
	}
	

	iSocket.SetOpt(KSoTcpReadUrgentData, KSolInetTcp, 1);
	iSocket.SetOpt(KSONonBlockingIO, KSOLSocket, 1);
	return 1;
}

TInt CGpfceEmulator::NetworkRecvDataL(TUint8 *data, TUint32 len, TInt block)
{
	int NoWaiting;
	if(block)
	 {
		 int t;
		 //uint8 temp[32];
		 TBuf8<32> temp;
		 TPtr8 tempptr(data, len);
		 //t=recv(Socket,temp,32,MSG_PEEK|MSG_DONTWAIT);
		 TRequestStatus status = KRequestPending;
		 iSocket.Recv(temp,KSockReadPeek, status);
		 User::WaitForRequest(status);
		 if(status != KErrNone)
		 {
			 if(status!=KErrWouldBlock) return(0);
		 }
		 else if(temp.Length() == 32)
			 NoWaiting|=2;
		 else
			 NoWaiting&=~2;
		 /*return(recv(Socket,data,len,0)==len);
		 */
		
		 status = KRequestPending;
		 iSocket.Recv(tempptr,0, status);
		 User::WaitForRequest(status);
		
		 return (status != KErrNone || tempptr.Length() != len)?0:1;
	 }
	 else
	 {
		 TRequestStatus status = KRequestPending;
		 TPtr8 temp(data, len);
		 iSocket.Recv(temp,0, status);
		 User::WaitForRequest(status);
		 /*int t=recv(Socket,data,len,MSG_DONTWAIT);*/
		 if(status != KErrNone)
		 {
			 if(status ==KErrWouldBlock)   // Would block
				 return(-1);
			 return(0);
		 }
		 return(1);
		 //return status != KErrNone?0:1;
	 }
	return 1;
}

TInt CGpfceEmulator::NetworkSendDataL(TUint8 *Value, TUint32 len)
{
	TPtrC8 ptr(Value, len);
	TRequestStatus status = KRequestPending;
	iSocket.Send(ptr,KSoTcpNextSendUrgentData, status);
	User::WaitForRequest(status);

	return status.Int() == KErrNone?1:0;
	//	 return(send(Socket,Value,len,0)==len);
}

void CGpfceEmulator::NetworkCloseL()
{
}

void CGpfceEmulator::UpdateInputL()
{
	iIdleCallBack->Cancel();
	iIdleCallBack->CallBack();
	CActiveScheduler::Start();
}
	
void CGpfceEmulator::Restart(RDirectScreenAccess::TTerminationReasons /*aReason*/)
{
	if(iForeground && iDsa != NULL)
	{
		iDsa->Cancel();
		iDsa->StartL();
		iDsa->Gc()->SetClippingRegion(iDsa->DrawingRegion());
	
		iDsa->Gc()->BitBlt(TPoint(0,0), iBackBuffer, iBackBuffer->SizeInPixels());
		iDsa->ScreenDevice()->Update();
	}
	
	iDrawingOn=ETrue;
}

void CGpfceEmulator::AbortNow(RDirectScreenAccess::TTerminationReasons /*aReason*/)
{
	if(iDsa != NULL)
	{
		iDsa->Cancel();
	}
	iDrawingOn=EFalse;
}


/**
 // a) Check QVGA screen
 // b) Check orientation
 // c) Check stretch or no stretch
 */

void UpdateScreenData()
{
	gDrawnWidth = 256;
	gAddition = 320-gDrawnWidth;//64; // 320 is the width of the drawn buffer
	gBufferOffset = 0;
	TPoint pos;
	TSize size;
	if(Settings.orientation == 0)
	{
		gOffset  = gScreenSize.iWidth*((gScreenSize.iHeight-240)/2);
		pos.iY = (gScreenSize.iHeight-240)/2;
		if(gDrawnWidth<gScreenSize.iWidth)
		{
			gOffset+=((gScreenSize.iWidth-gDrawnWidth)/2);	
			pos.iX = ((gScreenSize.iWidth-gDrawnWidth)/2);
		
		}
		else if(gDrawnWidth>gScreenSize.iWidth)
		{
						
			pos.iX = 0;
		
			gDrawnWidth = gScreenSize.iWidth;
			gBufferOffset+=((256-gDrawnWidth)/2);	
			gAddition+=(256-gDrawnWidth);		
			//aScreenBufferPtr+=((aScreenSize.iWidth-gDrawnWidth)/2);
		}
		size = TSize(gDrawnWidth, 240);
	}
	else
	{
		gOffset = (gScreenSize.iWidth-240)/2;
		pos.iX = (gScreenSize.iWidth-240)/2;
		if(gDrawnWidth>gScreenSize.iHeight)
		{
			gBufferOffset+=((gDrawnWidth-gScreenSize.iHeight)/2);
			pos.iY = 0;
			gAddition+=(gDrawnWidth-gScreenSize.iHeight);
			gDrawnWidth = gScreenSize.iHeight;
		}
		else if(gDrawnWidth<gScreenSize.iHeight)
		{
			gOffset+=(gScreenSize.iWidth*((gScreenSize.iHeight-gDrawnWidth)/2));
			pos.iY = ((gScreenSize.iHeight-gDrawnWidth)/2);
			
			//aScreenBufferPtr+=aScreenSize.iWidth*((aScreenSize.iHeight-gDrawnWidth)/2);
		}	
		size = TSize(240, gDrawnWidth);
	}

	gUpdateRect = TRect(pos, size);


}

inline void DrawPortraitNormal(TUint16* aScreenBufferPtr, TInt aScreenWidth)
{
//	TUint16* scrPtr = (TUint16*) iBackBuffer->DataAddress();
	TUint16* scrLinePtr = NULL;
	TUint8* buffer = (TUint8*) symbian_screen+32;
	
	aScreenBufferPtr+=gOffset;
	buffer+=gBufferOffset;
	
	for(TInt y= 0;y<240;y++)
	{
		scrLinePtr= aScreenBufferPtr;
#ifdef __WINS__
		for(TInt x=0;x<gDrawnWidth;x++)
		{
			*scrLinePtr = symbian_palette16[*buffer];			
			scrLinePtr++;
			buffer++;
		}
#else	
		convert2RGB555(scrLinePtr, buffer, symbian_palette16, gDrawnWidth);
		buffer+=gDrawnWidth;	
#endif
		buffer+=gAddition;
		
		aScreenBufferPtr+=aScreenWidth;		
	}
}

/**
 // a) Check QVGA screen
 // b) Check orientation
 // c) Check stretch or no stretch
 */

inline void DrawLandscapeNormal(TUint16* aScreenBufferPtr, TInt aScreenWidth, TBool aFlipped)
{
//	TUint16* scrPtr = (TUint16*) iBackBuffer->DataAddress();
	TUint16* scrLinePtr = NULL;
	TUint8* buffer = (TUint8*) symbian_screen+32;

	aScreenBufferPtr+=gOffset;
	buffer+=gBufferOffset;

	TInt nextLine = aFlipped?aScreenWidth:-aScreenWidth;	

	TInt incScrPtr = 1;
	if(aFlipped)
	{
		aScreenBufferPtr+=239; // Top line is +239
		incScrPtr = -1;
	}
	else
	{
		aScreenBufferPtr+=((gDrawnWidth-1)*aScreenWidth);
	}	
	
	for(TInt y= 0;y<240;y++)
	{
		scrLinePtr= aScreenBufferPtr;
#ifdef __WINS__
		for(TInt x=0;x<gDrawnWidth;x++)
		{
			*scrLinePtr = symbian_palette16[*buffer];			
			scrLinePtr+=nextLine;
			buffer++;;
		}
#else	
	for(TInt x=0;x<gDrawnWidth;x++)
		{
			*scrLinePtr = symbian_palette16[*buffer];			
			scrLinePtr+=nextLine;
			buffer++;;
		}
	/*
		convert2RGB555(scrLinePtr, buffer, symbian_palette16, drawnWidth);
		buffer+=drawnWidth;	*/
#endif
		buffer+=gAddition;
		
		aScreenBufferPtr+=incScrPtr;		
	}
}

void CGpfceEmulator::ClearScreen()
{
	if(iDrawingOn)
	{
		iBackBuffer->LockHeap();
		TSize size = iBackBuffer->SizeInPixels();
#ifdef __WINS__
		memset(iBackBuffer->DataAddress(),0, size.iWidth*size.iHeight);
#else
		memset32((int*) iBackBuffer->DataAddress(),0, (320*240*2)/4);
#endif
		iBackBuffer->UnlockHeap();
		iDsa->Gc()->SetBrushStyle(CGraphicsContext::ESolidBrush);
		iDsa->Gc()->SetBrushColor(KRgbBlack);
		iDsa->Gc()->Clear();
		iDsa->ScreenDevice()->Update();
	}
}

void CGpfceEmulator::UpdateScreenL()
{
	iBackBuffer->LockHeap();
	TSize size = iBackBuffer->SizeInPixels();
	if(running_emu)
	{
		if(Settings.stretch)
		{
			switch(Settings.orientation)
			{
				// Portrait mode
			case 0:
				{
					if(size.iWidth>=320)
					{
					}
					else
					{
					//DrawPortraitStretch((TUint16*) iBackBuffer->DataAddress(), size);
					}
				}break;
				// Landscape modes
			case 2:
			case 1:
				{
				}break;
				
			}
		}
		else
		{
			switch(Settings.orientation)
			{
				// Portrait mode
			case 0:
				{
					DrawPortraitNormal((TUint16*) iBackBuffer->DataAddress(), size.iWidth);
				}break;
				// Landscape modes
			case 2:
			case 1:
				{
					DrawLandscapeNormal((TUint16*) iBackBuffer->DataAddress(),size.iWidth, Settings.orientation == 2);
				}break;
				
			}
		}				
	}
	else
	{
		TUint16* scrPtr = (TUint16*) iBackBuffer->DataAddress();
		TUint16* scrLinePtr = NULL;		
		TUint16* buffer = (TUint16*) symbian_screen;
		TInt offset = (iBackBuffer->SizeInPixels().iHeight-240)/2;
		scrPtr+=(offset*iBackBuffer->SizeInPixels().iWidth);
		TInt drawnWidth = 320;//256;
		TInt addition = 320-drawnWidth;;
		if(drawnWidth>iBackBuffer->SizeInPixels().iWidth)
		{
			addition+=(drawnWidth-iBackBuffer->SizeInPixels().iWidth);
			drawnWidth = iBackBuffer->SizeInPixels().iWidth;
		}
		
		for(TInt y= 0;y<240;y++)
		{
			scrLinePtr= scrPtr;
			for(TInt x=0;x<drawnWidth;x++)
			{
				*scrLinePtr = *buffer;
				scrLinePtr++;
				buffer++;
			}
			buffer+=addition;
			scrPtr+=iBackBuffer->SizeInPixels().iWidth;
			
		}
	}
	iBackBuffer->UnlockHeap();

	if(iDrawingOn)
	{
		iDsa->Gc()->BitBlt(gUpdateRect.iTl, iBackBuffer, gUpdateRect);
		RRegion region;
		region.AddRect(gUpdateRect);
		iDsa->ScreenDevice()->Update(region);
		region.Close();
	}
}

void CGpfceEmulator::GetKeyStatusL(TUint16& aPlayer1, TUint16& aPlayer2)
{
	iAppUi.GetKeyStatusL(aPlayer1, aPlayer2);	
}

void CGpfceEmulator::GetMouseStatusL(TUint8& aX, TUint8& aY)
{
	aX = iLastPoint.iX;
	aY= iLastPoint.iY;
	iLastPoint.iX =255;
	iLastPoint.iY =255;
}

