/* FCE Ultra - NES/Famicom Emulator
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "../../state.h"
#include "../../general.h"
#include "../../input.h"
#include "../../video.h"
#include "..\..\driver.h"
#include "..\..\fce.h"
#include "symbian-runtime.h"
/* UsrInputType[] is user-specified.  InputType[] is current
       (game loading can override user settings)
*/
static int UsrInputType[2]={SI_GAMEPAD,SI_GAMEPAD};
static int UsrInputTypeFC={SI_NONE};

static int InputType[2];
static int InputTypeFC;

static uint32 JSreturn;

static int powerpadsc[2][12];
static int powerpadside=0;

static uint32 MouseData[3];
static uint8 fkbkeys[0x48];

int eoptions=0;
int waiting_for_input = 0;

static void do_emu_acts(uint32 aKeyState)
{
	if(!waiting_for_input)
	{
		waiting_for_input = 1;
		if (aKeyState & 0xC00)
		{
			unsigned long keys;
			int do_it = 1;
			if (aKeyState & 0x800)
			{
				if (Settings.sstate_confirm & 2)
				{
					FCEU_DispMessage("LOAD STATE? (C=no)");
					FCEU_PutImage();
					FCEUD_Update(XBuf+8,NULL,0);
					
					while( !((keys = symbian_input_read()) & (SYMBIAN_SELECT2|SYMBIAN_SELECT3)) ) usleep(50*1024);
					
					if (keys & SYMBIAN_SELECT2) do_it = 0;
					FCEU_CancelDispMessage();
					ClearScreen(emulator_pointer);
				}
				if (do_it) FCEUI_LoadState();
			}
			else
			{
				if (Settings.sstate_confirm & 1)
				{
					char *fname = FCEU_MakeFName(FCEUMKF_STATE,CurrentState,0);
					FILE *st=fopen(fname,"rb");
					free(fname);
					if (st)
					{
						fclose(st);
						FCEU_DispMessage("OVERWRITE SAVE?(C = no)");
						FCEU_PutImage();
						FCEUD_Update(XBuf+8,NULL,0);
						
						while( !((keys = symbian_input_read()) & (SYMBIAN_SELECT2|SYMBIAN_SELECT3)) ) usleep(50*1024);
						
						if (keys & SYMBIAN_SELECT2) do_it = 0;
						FCEU_CancelDispMessage();
						ClearScreen(emulator_pointer);
					}
				}
				if (do_it) FCEUI_SaveState();
			}
			RefreshThrottleFPS();
		}
		else if (aKeyState &0x3000) // state slot next/prev
		{
			FILE *st;
			char *fname;
			
			CurrentState += (aKeyState&0x1000) ? 1 :  -1;
			if (CurrentState > 9) CurrentState = 0;
			if (CurrentState < 0) CurrentState = 9;
			
			fname = FCEU_MakeFName(FCEUMKF_STATE,CurrentState,0);
			st=fopen(fname,"rb");
			free(fname);
			FCEU_DispMessage("[%s] State Slot %i", st ? "USED" : "FREE", CurrentState);
			if (st) fclose(st);
		}
		else if ((aKeyState & 0xc000) == 0xc000) // VS Unisystem insert coin
		{
			if(FCEUGameInfo.type == GIT_VSUNI)
				FCEU_DoSimpleCommand(FCEUNPCMD_VSUNICOIN);
		}
		else if (aKeyState & 0x4000) // FDS insert/eject
		{
			if(FCEUGameInfo.type == GIT_FDS)
				FCEU_DoSimpleCommand(FCEUNPCMD_FDSINSERT);
		}
		else if (aKeyState & 0x8000) // FDS select
		{
			if(FCEUGameInfo.type == GIT_FDS)
				FCEU_DoSimpleCommand(FCEUNPCMD_FDSSELECT);
		}
		
		waiting_for_input = 0;
	}	
}

static void do_fake_mouse(unsigned long keys)
{
	unsigned char mouseX, mouseY;
	static int x=256/2, y=240/2;
	int speed = 3;
	GetMouseStatusL(emulator_pointer, &mouseX, &mouseY);
	
	if(mouseX!= 255 && mouseY!=255)
	{
		x = mouseX;
		y = mouseY;
	}

	if (keys&0x100) speed = 1;
	if (keys&0x200) speed = 5;

	if (keys & 0x40)
	{
		x -= speed;
		if (x < 0) x = 0;
	}
	else if (keys & 0x80)
	{
		x += speed;
		if (x > 255) x = 255;
	}

	if (keys & 0x10)
	{
		y -= speed;
		if (y < 0) y = 0;
	}
	else if (keys & 0x20)
	{
		y += speed;
		if (y > 239) y = 239;
	}

	MouseData[0] = x;
	MouseData[1] = y;
	MouseData[2] = 0;
	if (keys&0x2) MouseData[2] |= 1;
	if (keys&0x1) MouseData[2] |= 2;
}

static void FCEUD_UpdateInput(void)
{
	unsigned short int player1 = 0, player2 = 0;
	static int volpushed_frames = 0;
	static int turbo_rate_cnt_a[2] = {0,0}, turbo_rate_cnt_b[2] = {0,0};
	uint32 all_acts[2] = {0,0};
	int i;
	GetKeyStatusL(emulator_pointer, &player1, &player2);
	all_acts[0] = player1;
	all_acts[1] = player2;

	JSreturn = 0; // RLDU SEBA

	if (InputType[1] != SI_GAMEPAD)
	{
		// try to feed fake mouse there 
		do_fake_mouse(player1);	
	}

	/*for (i = 0; i < 32; i++)
	{
		if (keys & (1 << i))
		{
			uint32 acts, u = 32;
			acts = Settings.KeyBinds[i];
			if (!acts) continue;
			if ((1 << i) & combo_keys)
			{
				// combo key detected, try to find if other is pressed
				for (u = i+1; u < 32; u++)
				{
					if ((keys & (1 << u)) && (Settings.KeyBinds[u] & acts))
					{
						keys &= ~(1 << u);
						acts &= Settings.KeyBinds[u];
						break;
					}
				}
			}
			if (u != 32) acts &=  combo_acts; // other combo key pressed
			else         acts &= ~combo_acts;
			all_acts[(acts>>16)&1] |= acts;
		}
	}*/

	// player 1
	JSreturn |= all_acts[0] & 0xff;
	if (all_acts[0] & 0x100) {		// A turbo
		turbo_rate_cnt_a[0] += Settings.turbo_rate_add;
		JSreturn |= (turbo_rate_cnt_a[0] >> 24) & 1;
	}
	if (all_acts[0] & 0x200) {		// B turbo
		turbo_rate_cnt_b[0] += Settings.turbo_rate_add;
		JSreturn |= (turbo_rate_cnt_b[0] >> 23) & 2;
	}

	// player 2
	JSreturn |= (all_acts[1] & 0xff) << 16;
	if (all_acts[1] & 0x100) {		// A turbo
		turbo_rate_cnt_a[1] += Settings.turbo_rate_add;
		JSreturn |= (turbo_rate_cnt_a[1] >> 8) & 0x10000;
	}
	if (all_acts[1] & 0x200) {		// B turbo
		turbo_rate_cnt_b[1] += Settings.turbo_rate_add;
		JSreturn |= (turbo_rate_cnt_b[1] >> 7) & 0x20000;
	}

	do_emu_acts(player1);
}


static void InitOtherInput(void)
{

   void *InputDPtr;

   int t;
   int x;
   int attrib;

   FCEU_printf("InitOtherInput: InputType[0]: %i, InputType[1]: %i, InputTypeFC: %i\n",
   	InputType[0], InputType[1], InputTypeFC);

   for(t=0,x=0;x<2;x++)
   {
    attrib=0;
    InputDPtr=0;
    switch(InputType[x])
    {
      //case SI_POWERPAD:InputDPtr=&powerpadbuf[x];break;
     case SI_GAMEPAD:InputDPtr=((uint8 *)&JSreturn)+(x<<1);break;
     case SI_ARKANOID:InputDPtr=MouseData;t|=1;break;
     case SI_ZAPPER:InputDPtr=MouseData;
                                t|=1;
                                attrib=1;
                                break;
    }
    FCEUI_SetInput(x,InputType[x],InputDPtr,attrib);
   }

   attrib=0;
   InputDPtr=0;
   switch(InputTypeFC)
   {
    case SIFC_SHADOW:InputDPtr=MouseData;t|=1;attrib=1;break;
    case SIFC_ARKANOID:InputDPtr=MouseData;t|=1;break;
    case SIFC_FKB:InputDPtr=fkbkeys;break;
   }

   FCEUI_SetInputFC(InputTypeFC,InputDPtr,attrib);
   FCEUI_DisableFourScore(eoptions&EO_NOFOURSCORE);

   inited|=16;
}


