// menu system for gpfce - FCE Ultra port
// (c) Copyright 2006,2007 notaz, All rights reserved.

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <dirent.h>
#include "driver.h"
#include "asmutils.h"
#include "symbian-menu.h"
#include "symbian-runtime.h"
#include "fonts.h"

#include "../../input.h"
#include "../../state.h"
#include "../../palette.h"
//#include "readpng.h"

int SYMBIAN_PORT_REV = 2;
extern int soundvol;
extern uint8 Exit; // exit emu loop flag
extern int InitSound(void);

static char path_buffer[256];
static unsigned short *menu_bg = 0;
static int txt_xmin, txt_xmax, txt_ymin, txt_ymax;

char menuErrorMsg[40] = {0, };
extern void log(int val);

int last_key;
int last_line = -1;
unsigned char mouseX, mouseY;
static const int screen_width_max = 240;
static const int screen_height_max = 240;
short int symbian_input_read()
{
	unsigned char cumouseX, cumouseY;

	last_key = 0;
	last_line = -1;
	Update_input(emulator_pointer);
	GetMouseStatusL(emulator_pointer, &cumouseX, &cumouseY);
	
	if(cumouseX != 255 && cumouseY != 255)
	{
		if(cumouseY>=txt_ymin && cumouseY<=txt_ymax)
		{
		last_line =(cumouseY-txt_ymin)/10;	
		}
	}

	return last_key;
}
static void menu_flip(void)
{
	UpdateScreenL(emulator_pointer);
}

static void menu_darken_reset(void)
{
	txt_xmin = 320; txt_xmax = 0;
	txt_ymin = 240; txt_ymax = 0;
}

static void gp2x_fceu_copy_bg(void)
{
	if (menu_bg)
	     memcpy(symbian_screen, menu_bg, 320*240*2);
	else memset(symbian_screen, 0, 320*240*2);
	menu_darken_reset();
}

static void menu_darken_text_bg(void)
{
	int x, y, xmin, xmax, ymax;
	unsigned short *screen = (unsigned short*) symbian_screen;

	xmin = txt_xmin - 3;
	if (xmin < 0) xmin = 0;
	xmax = txt_xmax + 2;
	if (xmax > 319) xmax = 319;

	y = txt_ymin - 3;
	if (y < 0) y = 0;
	ymax = txt_ymax + 2;
	if (ymax > 239) ymax = 239;

	for (x = xmin; x <= xmax; x++)
		screen[y*320+x] = 0xa514;
	for (y++; y < ymax; y++)
	{
		screen[y*320+xmin] = 0xffff;
		for (x = xmin+1; x < xmax; x++)
		{
			unsigned int p = screen[y*320+x];
			if (p != 0xffff)
				screen[y*320+x] = ((p&0xf79e)>>1) - ((p&0xc618)>>3);
		}
		screen[y*320+xmax] = 0xffff;
	}
	for (x = xmin; x <= xmax; x++)
		screen[y*320+x] = 0xffff;
}

static void gp2x_fceu_darken_all(void)
{
	unsigned int *screen = (unsigned int*) symbian_screen;
	int count = 320*240/2;

	while (count--)
	{
		unsigned int p = screen[count];
		screen[count] = ((p&0xf79ef79e)>>1) - ((p&0xc618c618)>>3);
	}
}

// draws white text to current bbp15 screen
static void gp2x_text_out15_(int x, int y, const char *text)
{
	int i,l;
	unsigned short *screen = (unsigned short*)symbian_screen;

	screen = screen + x + y*320;

	for (i = 0; i < strlen(text); i++)
	{
		for (l=0;l<8;l++)
		{
			if(fontdata8x8[((text[i])*8)+l]&0x80) screen[l*320+0]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x40) screen[l*320+1]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x20) screen[l*320+2]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x10) screen[l*320+3]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x08) screen[l*320+4]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x04) screen[l*320+5]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x02) screen[l*320+6]=0xffff;
			if(fontdata8x8[((text[i])*8)+l]&0x01) screen[l*320+7]=0xffff;
		}
		screen += 8;
	}
	if (x < txt_xmin) txt_xmin = x;
	if (x+i*8 > txt_xmax) txt_xmax = x+i*8;
	if (y < txt_ymin) txt_ymin = y;
	if (y+8   > txt_ymax) txt_ymax = y+8;
}

void gp2x_text_out15(int x, int y, const char *texto, ...)
{
	va_list args;
	char    buffer[512];

	va_start(args,texto);
	vsprintf(buffer,texto,args);
	va_end(args);

	gp2x_text_out15_(x,y,buffer);
}


void gp2x_text_out15_lim(int x, int y, const char *texto, int max)
{
	char    buffer[320/8+1];

	strncpy(buffer, texto, 320/8);
	if (max > 320/8) max = 320/8;
	if (max < 0) max = 0;
	buffer[max] = 0;

	gp2x_text_out15(x,y,buffer);
}

static void gp2x_smalltext16(int x, int y, const char *texto, unsigned short color)
{
	int i;
	unsigned char  *src;
	unsigned short *dst;

	for (i = 0;; i++, x += 6)
	{
		unsigned char c = (unsigned char) texto[i];
		int h = 8;

		if (!c) break;

		src = fontdata6x8[c];
		dst = (unsigned short *)symbian_screen + x + y*320;

		while (h--)
		{
			int w = 0x20;
			while (w)
			{
				if( *src & w ) *dst = color;
				dst++;
				w>>=1;
			}
			src++;

			dst += 320-6;
		}
	}
}

static void gp2x_smalltext16_lim(int x, int y, const char *texto, unsigned short color, int max)
{
	char    buffer[320/6+1];

	strncpy(buffer, texto, 320/6);
	if (max > 320/6) max = 320/6;
	if (max < 0) max = 0;
	buffer[max] = 0;

	gp2x_smalltext16(x, y, buffer, color);
}


static unsigned long inp_prev = 0;
static int inp_prevjoy = 0;

static unsigned long wait_for_input(unsigned long interesting)
{
	unsigned long ret;
	static int repeats = 0, wait = 50*1000;
	int release = 0, i;

	if (repeats == 2 || repeats == 4) wait /= 2;
	if (repeats == 6) wait = 15 * 1000;

	for (i = 0; i < 6 && inp_prev == symbian_input_read(); i++) {
		if (i == 0) repeats++;
		if (wait >= 30*1000) usleep(wait); // usleep sleeps for ~30ms minimum
	}

	while ( !((ret = symbian_input_read()) & interesting) ) {
		usleep(50000);
		release = 1;
	}

	if (release || ret != inp_prev) {
		repeats = 0;
		wait = 50*1000;
	}
	inp_prev = ret;
	inp_prevjoy = 0;

	// we don't need diagonals in menus
	if ((ret&SYMBIAN_UP)   && (ret&SYMBIAN_LEFT))  ret &= ~SYMBIAN_LEFT;
	if ((ret&SYMBIAN_UP)   && (ret&SYMBIAN_RIGHT)) ret &= ~SYMBIAN_RIGHT;
	if ((ret&SYMBIAN_DOWN) && (ret&SYMBIAN_LEFT))  ret &= ~SYMBIAN_LEFT;
	if ((ret&SYMBIAN_DOWN) && (ret&SYMBIAN_RIGHT)) ret &= ~SYMBIAN_RIGHT;

	return ret;
}

static unsigned long input2_read(unsigned long interesting, int *joy)
{
	unsigned long ret;

	do
	{
		*joy = 0;
		if ((ret = symbian_input_read() & interesting)) break;
		if (ret) break;
	}
	while(0);

	return ret;
}

// similar to wait_for_input(), but returns joy num
static unsigned long wait_for_input_usbjoy(unsigned long interesting, int *joy)
{
	unsigned long ret;
	const int wait = 300*1000;
	int i;

	if (inp_prevjoy == 0) inp_prev &= interesting;
	for (i = 0; i < 6; i++) {
		ret = input2_read(interesting, joy);
		if (*joy != inp_prevjoy || ret != inp_prev) break;
		usleep(wait/6);
	}

	while ( !(ret = input2_read(interesting, joy)) ) {
		usleep(50000);
	}

	inp_prev = ret;
	inp_prevjoy = *joy;

	return ret;
}



// -------------- ROM selector --------------

// rrrr rggg gggb bbbb
static unsigned short file2color(const char *fname)
{
	const char *ext = fname + strlen(fname) - 3;
	static const char *rom_exts[]   = { "zip", "nes", "fds", "unf", "nez", "nif" }; // nif is for unif
	static const char *other_exts[] = { "nsf", "ips", "fcm" };
	int i;

	if (ext < fname) ext = fname;
	for (i = 0; i < sizeof(rom_exts)/sizeof(rom_exts[0]); i++)
		if (strcasecmp(ext, rom_exts[i]) == 0) return 0xbdff;
	for (i = 0; i < sizeof(other_exts)/sizeof(other_exts[0]); i++)
		if (strcasecmp(ext, other_exts[i]) == 0) return 0xaff5;
	return 0xffff;
}

static void draw_dirlist(char *curdir, struct SymbianDirent **namelist, int n, int sel)
{
	int start, i, pos;

	start = 12 - sel;

	gp2x_fceu_copy_bg();
	gp2x_fceu_darken_all();

	if(start - 2 >= 0)
		gp2x_smalltext16_lim(14, (start - 2)*10, curdir, 0xffff, 53-2);
	for (i = 0; i < n; i++) {
		pos = start + i;
		if (pos < 0)  continue;
		if (pos > 23) break;
		if (namelist[i]->d_type == DT_DIR) {
			if(strlen(namelist[i]->d_name) > 1 && namelist[i]->d_name[1] == ':')
			{
			gp2x_smalltext16_lim(14, pos*10, namelist[i]->d_name, 0xfff6, 53-3);
			}
			else
			{
			gp2x_smalltext16_lim(14,   pos*10, "\\", 0xfff6, 1);
			gp2x_smalltext16_lim(14+6, pos*10, namelist[i]->d_name, 0xfff6, 53-3);
			}
		} else {
			unsigned short color = file2color(namelist[i]->d_name);
			gp2x_smalltext16_lim(14,   pos*10, namelist[i]->d_name, color, 53-2);
		}
	}
	gp2x_text_out15(5, 120, ">");
	menu_flip();
}

static int scandir_cmp(const void *p1, const void *p2)
{
	struct SymbianDirent **d1 = (struct SymbianDirent **)p1, **d2 = (struct SymbianDirent **)p2;
	if ((*d1)->d_type == (*d2)->d_type) return strcmp((*d1)->d_name, (*d1)->d_name);
	if ((*d1)->d_type == DT_DIR) return -1; // put before
	if ((*d2)->d_type == DT_DIR) return  1;
	return strcmp((*d1)->d_name, (*d1)->d_name);
}

static char *filter_exts[] = {
	".gpe", ".png", "ck.o", ".txt", ".srm"
};

static int scandir_filter(struct SymbianDirent *ent)
{
	const char *p;
	int i;

	if (ent == NULL || ent->d_name == NULL) return 0;
	if (strlen(ent->d_name) < 5) return 1;

	p = ent->d_name + strlen(ent->d_name) - 4;

	for (i = 0; i < sizeof(filter_exts)/sizeof(filter_exts[0]); i++)
	{
		if (strcmp(p, filter_exts[i]) == 0) return 0;
	}

	return 1;
}

static char *filesel_loop(char *curr_path, char *final_dest)
{
	struct SymbianDirent **namelist;
	DIR *dir;
	int n, newlen, sel = 0;
	unsigned long inp = 0;
	char *ret = NULL, *fname = NULL;

	// is this a dir or a full path?
	if ((dir = opendir(curr_path))) {
		closedir(dir);
	} else {
		char *p;
		for (p = curr_path + strlen(curr_path) - 1; p > curr_path && *p != '/' && *p != '\\'; p--);
		*p = 0;
		fname = p+1;
	}

	n = SymbianScandir(curr_path, &namelist, scandir_filter, scandir_cmp);
	if (n < 0) {
		// try root
		n = SymbianScandir("", &namelist, scandir_filter, scandir_cmp);
		if (n < 0) {
			// oops, we failed
			printf("dir: %s\n", curr_path);
			perror("scandir");
			return NULL;
		}
		else
		{
			curr_path[0] = 0;
		}
	}

	// try to find sel
	if (fname != NULL) {
		int i;
		for (i = 0; i < n; i++) {
			if (strcmp(namelist[i]->d_name, fname) == 0) {
				sel = i;
				break;
			}
		}
	}

	for (;;)
	{
		draw_dirlist(curr_path, namelist, n, sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  )  { sel--;   if (sel < 0)   sel = n-1; }
		if(inp & SYMBIAN_DOWN)  { sel++;   if (sel > n-1) sel = 0; }
		if(inp & SYMBIAN_LEFT)  { sel-=10; if (sel < 0)   sel = 0; }
	//	if(inp & SYMBIAN_L)     { sel-=24; if (sel < 0)   sel = 0; }
		if(inp & SYMBIAN_RIGHT) { sel+=10; if (sel > n-1) sel = n-1; }
	//	if(inp & SYMBIAN_R)     { sel+=24; if (sel > n-2) sel = n-2; }
		if(inp & SYMBIAN_SELECT3)     { // enter dir/select
			again:
			newlen = strlen(curr_path) + strlen(namelist[sel]->d_name) + 2;
			if (namelist[sel]->d_type == DT_REG) { // file selected
				if (final_dest == NULL) final_dest = malloc(newlen);
				if (final_dest == NULL) break;
				strcpy(final_dest, curr_path);
				strcat(final_dest, "\\");
				strcat(final_dest, namelist[sel]->d_name);
				ret = final_dest;
				break;
			} else if (namelist[sel]->d_type == DT_DIR) {
				char *p, *newdir = malloc(newlen);
				if (newdir == NULL) break;
				if (strcmp(namelist[sel]->d_name, "..") == 0) {
					char *start = curr_path;
					p = start + strlen(start) - 1;
					while ((*p == '/'  || *p == '\\')&& p > start) p--;
					while (*p != '/' && *p != '\\' && p > start) p--;
					if (p <= start) strcpy(newdir, "/");
					else { strncpy(newdir, start, p-start); newdir[p-start] = 0; }
				} else {
					strcpy(newdir, curr_path);
					p = newdir + strlen(newdir) - 1;
					while (p >= newdir && (*p == '/' || *p == '\\')) *p-- = 0;
					if(strlen(namelist[sel]->d_name)>1 && namelist[sel]->d_name[1] != ':')
					{
						strcat(newdir, "\\");
					}

					strcat(newdir, namelist[sel]->d_name);
				}
				ret = filesel_loop(newdir, final_dest);
				free(newdir);
				break;
			} else {
				// unknown file type, happens on NTFS mounts. Try to guess.
				char *tstfn; FILE *tstf; int tmp;
				tstfn = malloc(newlen);
				if (tstfn == NULL) break;
				strcpy(tstfn, curr_path);
				strcat(tstfn, "\\");
				strcat(tstfn, namelist[sel]->d_name);
				tstf = fopen(tstfn, "rb");
				free(tstfn);
				if (tstf != NULL)
				{
					if (fread(&tmp, 1, 1, tstf) > 0 || ferror(tstf) == 0)
						namelist[sel]->d_type = DT_REG;
					else	namelist[sel]->d_type = DT_DIR;
					fclose(tstf);
					goto again;
				}
			}
		}
		if(inp & SYMBIAN_SELECT2) break; // cancel
	}

	if (n > 0) {
		while(n--) free(namelist[n]);
		free(namelist);
	}

	return ret;
}

// ------------ patch/gg menu ------------

extern void *cheats;
static int cheat_count = 0, cheat_start, cheat_pos;

static int countcallb(char *name, uint32 a, uint8 v, int compare, int s, int type, void *data)
{
	cheat_count++;
	return 1;
}

static int clistcallb(char *name, uint32 a, uint8 v, int compare, int s, int type, void *data)
{
	int pos;

	pos = cheat_start + cheat_pos;
	cheat_pos++;
	if (pos < 0)  return 1;
	if (pos > 23) return 0;

	gp2x_smalltext16_lim(14,     pos*10, s ? "ON " : "OFF", 0xffff, 3);
	gp2x_smalltext16_lim(14+6*4, pos*10, type ? "S" : "R", 0xffff, 1);
	gp2x_smalltext16_lim(14+6*6, pos*10, name, 0xffff, 53-8);

	return 1;
}

static void draw_patchlist(int sel)
{
	int pos;

	gp2x_fceu_copy_bg();
	gp2x_fceu_darken_all();

	cheat_start = 12 - sel;
	cheat_pos = 0;
	FCEUI_ListCheats(clistcallb,0);

	pos = cheat_start + cheat_pos;
	if (pos < 24) gp2x_smalltext16_lim(14, pos*10, "done", 0xffff, 4);

	gp2x_text_out15(5, 120, ">");
	menu_flip();
}

void patches_menu_loop(void)
{
	int menu_sel = 0;
	unsigned long inp = 0;

	cheat_count = 0;
	FCEUI_ListCheats(countcallb,0);

	for(;;)
	{
		draw_patchlist(menu_sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  ) { menu_sel--; if (menu_sel < 0) menu_sel = cheat_count; }
		if(inp & SYMBIAN_DOWN) { menu_sel++; if (menu_sel > cheat_count) menu_sel = 0; }
		if(inp &(SYMBIAN_LEFT))  { menu_sel-=10; if (menu_sel < 0) menu_sel = 0; }
		if(inp &(SYMBIAN_RIGHT)) { menu_sel+=10; if (menu_sel > cheat_count) menu_sel = cheat_count; }
		if(inp & SYMBIAN_SELECT3) { // action
			if (menu_sel < cheat_count)
			     FCEUI_ToggleCheat(menu_sel);
			else return;
		}
		if(inp & SYMBIAN_SELECT2) return;
	}

}

// ------------ savestate loader ------------

#if 0
static void menu_prepare_bg(void);

static int state_slot_flags = 0;

static void state_check_slots(void)
{
	int slot;

	state_slot_flags = 0;

	for (slot = 0; slot < 10; slot++)
	{
		if (emu_check_save_file(slot))
		{
			state_slot_flags |= 1 << slot;
		}
	}
}

static void draw_savestate_bg(int slot)
{
	struct PicoVideo tmp_pv;
	unsigned short tmp_cram[0x40];
	unsigned short tmp_vsram[0x40];
	void *tmp_vram, *file;
	char *fname;

	fname = emu_GetSaveFName(1, 0, slot);
	if (!fname) return;

	tmp_vram = malloc(sizeof(Pico.vram));
	if (tmp_vram == NULL) return;

	memcpy(tmp_vram, Pico.vram, sizeof(Pico.vram));
	memcpy(tmp_cram, Pico.cram, sizeof(Pico.cram));
	memcpy(tmp_vsram, Pico.vsram, sizeof(Pico.vsram));
	memcpy(&tmp_pv, &Pico.video, sizeof(Pico.video));

	if (strcmp(fname + strlen(fname) - 3, ".gz") == 0) {
		file = gzopen(fname, "rb");
		emu_set_save_cbs(1);
	} else {
		file = fopen(fname, "rb");
		emu_set_save_cbs(0);
	}

	if (file) {
		if (PicoMCD & 1) {
			PicoCdLoadStateGfx(file);
		} else {
			areaSeek(file, 0x10020, SEEK_SET);  // skip header and RAM in state file
			areaRead(Pico.vram, 1, sizeof(Pico.vram), file);
			areaSeek(file, 0x2000, SEEK_CUR);
			areaRead(Pico.cram, 1, sizeof(Pico.cram), file);
			areaRead(Pico.vsram, 1, sizeof(Pico.vsram), file);
			areaSeek(file, 0x221a0, SEEK_SET);
			areaRead(&Pico.video, 1, sizeof(Pico.video), file);
		}
		areaClose(file);
	}

	emu_forced_frame();
	gp2x_memcpy_buffers((1<<2), symbian_screen, 0, 320*240*2);
	menu_prepare_bg();

	memcpy(Pico.vram, tmp_vram, sizeof(Pico.vram));
	memcpy(Pico.cram, tmp_cram, sizeof(Pico.cram));
	memcpy(Pico.vsram, tmp_vsram, sizeof(Pico.vsram));
	memcpy(&Pico.video, &tmp_pv,  sizeof(Pico.video));
	free(tmp_vram);
}

static void draw_savestate_menu(int menu_sel, int is_loading)
{
	int tl_x = 15, tl_y = 60, y, i;

	if (state_slot_flags & (1 << menu_sel))
		draw_savestate_bg(menu_sel);
	gp2x_fceu_copy_bg();

	gp2x_text_out15(tl_x, 30, is_loading ? "Load state" : "Save state");

	/* draw all 10 slots */
	y = tl_y;
	for (i = 0; i < 10; i++, y+=10)
	{
		gp2x_text_out15(tl_x, y, "SLOT %i (%s)", i, (state_slot_flags & (1 << i)) ? "USED" : "free");
	}
	gp2x_text_out15(tl_x, y, "back");

	// draw cursor
	gp2x_text_out15(tl_x - 16, tl_y + menu_sel*10, ">");

	menu_flip();
}

static int savestate_menu_loop(int is_loading)
{
	int menu_sel = 10, menu_sel_max = 10;
	unsigned long inp = 0;

	state_check_slots();

	for(;;)
	{
		draw_savestate_menu(menu_sel, is_loading);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  ) {
			do {
				menu_sel--; if (menu_sel < 0) menu_sel = menu_sel_max;
			} while (!(state_slot_flags & (1 << menu_sel)) && menu_sel != menu_sel_max && is_loading);
		}
		if(inp & SYMBIAN_DOWN) {
			do {
				menu_sel++; if (menu_sel > menu_sel_max) menu_sel = 0;
			} while (!(state_slot_flags & (1 << menu_sel)) && menu_sel != menu_sel_max && is_loading);
		}
		if(inp & SYMBIAN_SELECT3) { // save/load
			if (menu_sel < 10) {
				state_slot = menu_sel;
				if (emu_SaveLoadGame(is_loading, 0)) {
					strcpy(menuErrorMsg, is_loading ? "Load failed" : "Save failed");
					return 1;
				}
				return 0;
			} else	return 1;
		}
		if(inp & SYMBIAN_SELECT2) return 1;
	}
}
#endif

typedef struct { char *name; int mask; } bind_action_t;

static void draw_key_config(const bind_action_t *opts, int opt_cnt, int player_idx, int sel)
{
	char scan_str[16];
	int x, y, tl_y = 40, i;
	int scan_offset = 0;

	if(player_idx == 1)
	{
		scan_offset = 10;
	}
	else if(player_idx == -1)
	{
		scan_offset = 20;
	}
	
	gp2x_fceu_copy_bg();

	if (sel < opt_cnt) {
		if(setting_key)
		{
			gp2x_text_out15(30, 190, "Press key for");
			gp2x_text_out15(30, 200,  opts[sel].name);
			gp2x_text_out15(30, 210, "action");
		}
		else
		{
			gp2x_text_out15(30, 190, "Press \"select\"");
			gp2x_text_out15(30, 200, "to bind action!");
			gp2x_text_out15(30, 210, "Press \"C\" to clear");
			gp2x_text_out15(30, 220, "Select \"Done\" to exit");
		}
	} else {
		gp2x_text_out15(30, 200, "Use Options -> Save cfg");
		gp2x_text_out15(30, 210, "to save controls");
		gp2x_text_out15(30, 220, "Press \"select\" to exit");
	}
	
	menu_darken_text_bg();
    
	menu_darken_reset();

	if (player_idx >= 0)
	     gp2x_text_out15(80, 20, "Player %i controls", player_idx + 1);
	else gp2x_text_out15(80, 20, "Emulator controls");

	x = 25; y = tl_y;

	for (i = 0; i < opt_cnt; i++, y+=10)
	{
		int scancode = Settings.ScanCodes[scan_offset+i];

		if(scancode<= 255 && scan_name[scancode] != NULL && strlen(scan_name[scancode]) >0)
		{
			strcpy(scan_str, scan_name[scancode]);
		}
		else
		{
			if(scancode>=(' '+1) && scancode<='Z')
			{
				sprintf(scan_str,"'%c'", scancode);
			}
			else
			{
				if(scancode != 0)
				{
					sprintf(scan_str,"0x%x", scancode);
				}
				else
				{
					sprintf(scan_str, "----");
				}
			}
		}

		gp2x_text_out15(x, y, "%s : %s", opts[i].name, scan_str/*action_binds(player_idx, opts[i].mask)*/);
	}

	gp2x_text_out15(x, y, "Done");

	// draw cursor
	gp2x_text_out15(x - 16, tl_y + sel*10, ">");

	menu_darken_text_bg();

	menu_flip();
}

static void key_config_loop(const bind_action_t *opts, int opt_cnt, int player_idx)
{
	int joy = 0, sel = 0, menu_sel_max = opt_cnt;
	unsigned long inp = 0;
	int scan_offset = 0;

	if(player_idx == 1)
	{
		scan_offset = 10;
	}
	else if(player_idx == -1)
	{
		scan_offset = 20;
	}

	for (;;)
	{
		draw_key_config(opts, opt_cnt, player_idx, sel);
		inp = wait_for_input_usbjoy(0xffff, &joy);
	
		if (joy == 0) {
		//	if (!(inp & SYMBIAN_VOL_DOWN)) {
				if(inp & SYMBIAN_UP  ) { sel--; if (sel < 0) sel = menu_sel_max; continue; }
				if(inp & SYMBIAN_DOWN) { sel++; if (sel > menu_sel_max) sel = 0; continue; }
			//}
			if (sel >= opt_cnt) {
				if (inp & (SYMBIAN_SELECT3|SYMBIAN_SELECT2)) break;
				else continue;
			}
	
			if (inp & (SYMBIAN_SELECT3)) 
			{
				setting_key = 1;
				setting_key_scancode = 0;
				draw_key_config(opts, opt_cnt, player_idx, sel);
				while(setting_key)
				{
					symbian_input_read();
					usleep(15000);
				}
				Settings.ScanCodes[sel+scan_offset] = setting_key_scancode;
			}

			if(inp & SYMBIAN_SELECT2)
			{
				Settings.ScanCodes[sel+scan_offset] = 0;
			}
			// if we are here, we want to bind/unbind something
		//	if (inp & SYMBIAN_VOL_UP)
		//		unbind_action(opts[sel].mask);
#pragma message("unbind")
			//inp &= CONFIGURABLE_KEYS;
		/*	for (i = 0; i < 32; i++)
				if (inp & (1 << i)) {
					if (count_bound_keys(opts[sel].mask, 0) >= 2)
					     Settings.KeyBinds[i] &= ~opts[sel].mask; // allow to unbind only
					else Settings.KeyBinds[i] ^=  opts[sel].mask;
					if (player_idx >= 0) {
						Settings.KeyBinds[i] &= ~(3 << 16);
						Settings.KeyBinds[i] |= player_idx << 16;
					}
				}*/
		}
/*		else if (sel < opt_cnt)
		{
			for (i = 0; i < 32; i++)
				if (inp & (1 << i)) {
					if (count_bound_keys(opts[sel].mask, joy) >= 1) // disallow combos for usbjoy
					     Settings.JoyBinds[joy-1][i] &= ~opts[sel].mask;
					else Settings.JoyBinds[joy-1][i] ^=  opts[sel].mask;
					if (player_idx >= 0) {
						Settings.JoyBinds[joy-1][i] &= ~(3 << 16);
						Settings.JoyBinds[joy-1][i] |= player_idx << 16;
					}
				}
		}*/
	}
}

static void draw_kc_sel(int menu_sel)
{
	int tl_x = 15+40, tl_y = 60, y;

	y = tl_y;
	gp2x_fceu_copy_bg();
	gp2x_text_out15(tl_x, y,       "Player 1");
	gp2x_text_out15(tl_x, (y+=10), "Player 2");
	gp2x_text_out15(tl_x, (y+=10), "Emulator controls");
	gp2x_text_out15(tl_x, (y+=10), "Done");

	// draw cursor
	gp2x_text_out15(tl_x - 16, tl_y + menu_sel*10, ">");

	menu_darken_text_bg();
	menu_flip();
}

// b_turbo,a_turbo  RLDU SEBA
static bind_action_t ctrl_actions[] =
{
	{ "UP     ", 0x010 },
	{ "DOWN   ", 0x020 },
	{ "LEFT   ", 0x040 },
	{ "RIGHT  ", 0x080 },
	{ "A      ", 0x001 },
	{ "B      ", 0x002 },
	{ "A TURBO", 0x100 },
	{ "B TURBO", 0x200 },
	{ "START  ", 0x008 },
	{ "SELECT ", 0x004 },
};

static bind_action_t emuctrl_actions[] =
{
	{ "Save State       ", 1<<31 },
	{ "Load State       ", 1<<30 },
	{ "Next State Slot  ", 1<<29 },
	{ "Prev State Slot  ", 1<<28 },
	{ "FDS Insert/Eject ", 1<<27 },
	{ "FDS Select Disk  ", 1<<26 },
	{ "VSUni Insert Coin", 1<<25 },
};

static void kc_sel_loop(void)
{
	int menu_sel = 3, menu_sel_max = 3;
	unsigned long inp = 0;

	for(;;)
	{
		draw_kc_sel(menu_sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  ) { menu_sel--; if (menu_sel < 0) menu_sel = menu_sel_max; }
		if(inp & SYMBIAN_DOWN) { menu_sel++; if (menu_sel > menu_sel_max) menu_sel = 0; }
		if(inp & SYMBIAN_SELECT3) {
			switch (menu_sel) {
				case 0: key_config_loop(ctrl_actions, 10, 0); return;
				case 1: key_config_loop(ctrl_actions, 10, 1); return;
				case 2: key_config_loop(emuctrl_actions,
						sizeof(emuctrl_actions)/sizeof(emuctrl_actions[0]), -1); return;
				case 3: if (!fceugi) SaveConfig(NULL); return;
				default: return;
			}
		}
		if(inp & SYMBIAN_SELECT2) return;
	}
}


// --------- FCEU options ----------

extern int ntsccol,ntschue,ntsctint;
extern int srendlinev[2];
extern int erendlinev[2];
extern int eoptions;
extern char *cpalette;
extern void LoadCPalette(void);


static void int_incdec(int *p, int inc, int min, int max)
{
	*p += inc;
	if      (*p < min) *p = min;
	else if (*p > max) *p = max;
}

static void draw_fcemenu_options(int menu_sel)
{
	int tl_x = 20, tl_y = 60, y;
	char cpal[32];
	
	gp2x_fceu_copy_bg();

	if (menu_sel == 0) {		

		gp2x_text_out15(30, 210, "Press B to browse,");
		gp2x_text_out15(30, 220, "START to use default");
		menu_darken_text_bg();

		menu_darken_reset();
	}

	if (cpalette != NULL)
	{
		char *p = cpalette + strlen(cpalette) - 1;
		while (*p != '/' && *p != '\\' && p > cpalette) p--;
		if (*p == '/' || *p == '\\') p++;
		strncpy(cpal, p, 16);
		cpal[16] = 0;
	}
	else strcpy(cpal, "         OFF");

	y = tl_y;


	gp2x_text_out15(tl_x,  y,      "Custom palette:%s", cpal);				// 0
	gp2x_text_out15(tl_x, (y+=10), "NTSC Color Emulation    %s", ntsccol?" ON":"OFF");
	gp2x_text_out15(tl_x, (y+=10), "  Tint (default: 56)   % 3d", ntsctint);
	gp2x_text_out15(tl_x, (y+=10), "  Hue  (default: 72)   % 3d", ntschue);
	gp2x_text_out15(tl_x, (y+=10), "First vis. line(NTSC)  % 3d", srendlinev[0]);
	gp2x_text_out15(tl_x, (y+=10), "Last visible line(NTSC)% 3d", erendlinev[0]);		// 5
	gp2x_text_out15(tl_x, (y+=10), "First visible line(PAL)% 3d", srendlinev[1]);
	gp2x_text_out15(tl_x, (y+=10), "Last visible line(PAL) % 3d", erendlinev[1]);
	gp2x_text_out15(tl_x, (y+=10), "Clip 8 left/right cols  %s", (eoptions&EO_CLIPSIDES)?" ON":"OFF");
	gp2x_text_out15(tl_x, (y+=10), "Disable 8 sprite limit  %s", (eoptions&EO_NO8LIM)?" ON":"OFF");
	gp2x_text_out15(tl_x, (y+=10), "Enable real GameGenie   %s", (eoptions&EO_GG)?" ON":"OFF");
	gp2x_text_out15(tl_x, (y+=10), "Done");							// 11

	// draw cursor
	gp2x_text_out15(tl_x - 16, tl_y + menu_sel*10, ">");


	menu_darken_text_bg();
	menu_flip();
}

static void fcemenu_loop_options(void)
{
	int menu_sel = 0, menu_sel_max = 11, i;
	unsigned long inp = 0;

	FCEUI_GetNTSCTH(&ntsctint, &ntschue);

	for(;;)
	{
		draw_fcemenu_options(menu_sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3|SYMBIAN_SELECT2|SYMBIAN_SELECT4);
		if(inp & SYMBIAN_UP  ) { menu_sel--; if (menu_sel < 0) menu_sel = menu_sel_max; }
		if(inp & SYMBIAN_DOWN) { menu_sel++; if (menu_sel > menu_sel_max) menu_sel = 0; }
		if((inp& SYMBIAN_SELECT3)||(inp&SYMBIAN_LEFT)||(inp&SYMBIAN_RIGHT)) { // toggleable options
			switch (menu_sel) {
				case  1: ntsccol = !ntsccol; break;
				case  8: eoptions^=EO_CLIPSIDES; break;
				case  9: eoptions^=EO_NO8LIM; break;
				case 10: eoptions^=EO_GG; break;
				case 11: return;
			}
		}
		if(inp & (SYMBIAN_SELECT2)) {
			for(i=0;i<2;i++)
			{
				if(srendlinev[i]<0 || srendlinev[i]>239) srendlinev[i]=0;
				if(erendlinev[i]<srendlinev[i] || erendlinev[i]>239) erendlinev[i]=239;
			}
			FCEUI_SetNTSCTH(ntsccol, ntsctint, ntschue);
			FCEUI_SetRenderedLines(srendlinev[0],erendlinev[0],srendlinev[1],erendlinev[1]);
			FCEUI_DisableSpriteLimitation(eoptions&EO_NO8LIM);
			FCEUI_SetGameGenie(eoptions&EO_GG);
			if (cpalette) LoadCPalette();
			else FCEUI_SetPaletteArray(0); // set to default
			FCEU_ResetPalette();
			return;
		}
		if(inp & (SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3)) { // multi choise
			switch (menu_sel) {
				case  2: int_incdec(&ntsctint,      (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 128); break;
				case  3: int_incdec(&ntschue,       (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 128); break;
				case  4: int_incdec(&srendlinev[0], (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 239); break;
				case  5: int_incdec(&erendlinev[0], (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 239); break;
				case  6: int_incdec(&srendlinev[1], (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 239); break;
				case  7: int_incdec(&erendlinev[1], (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 239); break;
			}
		}
		if(menu_sel == 0 && (inp & (SYMBIAN_SELECT4|SYMBIAN_SELECT3))) { // custom palette
			if ((inp & SYMBIAN_SELECT4) && cpalette) {
				free(cpalette);
				cpalette=NULL;
			}
			else if (inp & SYMBIAN_SELECT3) {
				char *selfname;
				if (cpalette) strncpy(path_buffer, cpalette, sizeof(path_buffer));
				else getcwd(path_buffer, 256);
				path_buffer[sizeof(path_buffer)-1] = 0;

				selfname = filesel_loop(path_buffer, NULL);
				if (selfname) {
					if (cpalette) free(cpalette);
					cpalette = selfname;
				}
			}
		}
	}
}

// -------------- Network options --------------
static void draw_network_options(int menu_sel)
{
	int tl_x = 20, tl_y = 20, y;

	y = tl_y;
	gp2x_fceu_copy_bg();

	gp2x_text_out15(tl_x,  y,"Network type: %s", Settings.networktype ? "Ip":"Bluetooth");				// 0
	gp2x_text_out15(tl_x, (y+=10), "Wait for connection");	// 1
	gp2x_text_out15(tl_x, (y+=10), "Connect to other client");

	// draw cursor
	gp2x_text_out15(tl_x - 16, tl_y + menu_sel*10, ">");

	menu_darken_text_bg();
	menu_flip();
}

void network_menu_loop(void)
{
	int menu_sel = 0;
	unsigned long inp = 0;

	for(;;)
	{
		draw_network_options(menu_sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  ) { menu_sel--; if (menu_sel < 0) menu_sel = 2; }
		if(inp & SYMBIAN_DOWN) { menu_sel++; if (menu_sel > 2) menu_sel = 0; }
		if(inp & SYMBIAN_SELECT3) { // action
			switch(menu_sel)
			{
			case 0:
				Settings.networktype = 1- Settings.networktype;
				break;
			case 1:
				FCEUI_SetNetworkPlay(1);
				return;
			case 2:
				FCEUI_SetNetworkPlay(2);
				return;
			}

		}
		if(inp & SYMBIAN_SELECT2) return;
	}

}

// -------------- options --------------

static void draw_menu_options(int menu_sel)
{
	int tl_x = 20, tl_y = 20, y;
	char strframeskip[8], *strscaling, *strssconfirm, *strorientation;

	if (Settings.frameskip < 0)
	     strcpy(strframeskip, "Auto");
	else sprintf(strframeskip, "% 4d", Settings.frameskip);

	switch (Settings.stretch) {
		default: strscaling = "OFF";   break;
		case 1:  strscaling = " ON";     break;	
	}

	switch (Settings.orientation) {
		default: strorientation = "  Portrait";   break;
		case 1:  strorientation = " Landscape";     break;
		case 2:  strorientation = "Landscape2"; break;		
	}

	switch (Settings.sstate_confirm) {
		default: strssconfirm = "   OFF";    break;
		case 1:  strssconfirm = "writes"; break;
		case 2:  strssconfirm = " loads";  break;
		case 3:  strssconfirm = "  both";   break;
	}

	y = tl_y;
	gp2x_fceu_copy_bg();

	gp2x_text_out15(tl_x,  y,      "Orientation:     %s", strorientation);				// 0
	gp2x_text_out15(tl_x, (y+=10), "Stretch:                %s", strscaling);				// 1
	gp2x_text_out15(tl_x, (y+=10), "Show FPS                %s", Settings.showfps?" ON":"OFF");	// 2
	gp2x_text_out15(tl_x, (y+=10), "Frameskip              %s", strframeskip);			// 3
	gp2x_text_out15(tl_x, (y+=10), "Accurate renderer(slow) %s", Settings.accurate_mode?" ON":"OFF");
	if(soundvol == 0)
	{
	gp2x_text_out15(tl_x, (y+=10), "Sound vol:              OFF");
	}
	else
	{
	gp2x_text_out15(tl_x, (y+=10), "Sound vol:             % 3d", soundvol);
	}

	gp2x_text_out15(tl_x, (y+=10), "Sound Rate:         %5iHz", Settings.sound_rate);		// 5
	gp2x_text_out15(tl_x, (y+=10), "Force Region:          %s",
		Settings.region_force == 2 ? " PAL" : Settings.region_force == 1 ? "NTSC" : " OFF");	// 6
	gp2x_text_out15(tl_x, (y+=10), "Turbo rate            % 2dHz", (Settings.turbo_rate_add*60/2) >> 24);
	gp2x_text_out15(tl_x, (y+=10), "Confirm savestate    %s", strssconfirm);			// 8
	gp2x_text_out15(tl_x, (y+=10), "Save slot                 %i", CurrentState);
//	gp2x_text_out15(tl_x, (y+=10), "Faster RAM timings         %s", Settings.ramtimings?" ON":"OFF");
	//gp2x_text_out15(tl_x, (y+=10), "squidgehack (now %s %s",   mms, Settings.mmuhack?" ON":"OFF");	// 11
	//gp2x_text_out15(tl_x, (y+=10), "Gamma correction           %i.%02i", Settings.gamma / 100, Settings.gamma%100);
	gp2x_text_out15(tl_x, (y+=10), "Perfect VSYNC           %s", Settings.perfect_vsync?" ON":"OFF");
//	gp2x_text_out15(tl_x, (y+=10), "GP2X CPU clock             %iMhz", Settings.cpuclock);		// 14
	gp2x_text_out15(tl_x, (y+=10), "[FCE Ultra options]");
	gp2x_text_out15(tl_x, (y+=10), "Save cfg as default");						// 16
	if (fceugi)
		gp2x_text_out15(tl_x, (y+=10), "Save cfg for curr game only");

	// draw cursor
	gp2x_text_out15(tl_x - 16, tl_y + menu_sel*10, ">");

	if (menu_sel == 3) {
		gp2x_text_out15(tl_x, 210-40, "ROM reload required for");
		gp2x_text_out15(tl_x, 220-40, "this setting to take effect");
	} /*else if (menu_sel == 10 || menu_sel == 11) {
		gp2x_text_out15(tl_x, 210-40, "Emu restart required for this");
		gp2x_text_out15(tl_x, 220-40, "setting to take effect");
	}*/

	menu_darken_text_bg();
	menu_flip();
}

static int sndrate_prevnext(int rate, int dir)
{
	int i, rates[] = { 8000, 11025, 16000, 22050, 44100 };

	for (i = 0; i < 5; i++)
		if (rates[i] == rate) break;

	i += dir ? 1 : -1;
	if (i > 4) return dir ? 44100 : 22050;
	if (i < 0) return dir ? 11025 : 8000;
	return rates[i];
}

static void config_commit(void)
{
	if (Settings.region_force)
		FCEUI_SetVidSystem(Settings.region_force - 1);
}

// Main options

static int menu_loop_options(void)
{
	static int menu_sel = 0;
	int ret, menu_sel_max = 13;//16;
	unsigned long inp = 0;

	if (fceugi) menu_sel_max++;

	for(;;)
	{
		draw_menu_options(menu_sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  ) { menu_sel--; if (menu_sel < 0) menu_sel = menu_sel_max; }
		if(inp & SYMBIAN_DOWN) { menu_sel++; if (menu_sel > menu_sel_max) menu_sel = 0; }
		if((inp& SYMBIAN_SELECT3)||(inp&SYMBIAN_LEFT)||(inp&SYMBIAN_RIGHT)) { // toggleable options
			switch (menu_sel) {
				case  1: Settings.stretch= !Settings.stretch; break;
				case  2: Settings.showfps       = !Settings.showfps; break;
				case  4: Settings.accurate_mode = !Settings.accurate_mode; break;
				case  5: soundvol = (soundvol == 100)? 0 : soundvol+10; break;
			//	case 10: Settings.ramtimings    = !Settings.ramtimings; break;
//				case 11: Settings.mmuhack       = !Settings.mmuhack; break;
				case 11/*13*/: Settings.perfect_vsync = !Settings.perfect_vsync; break;
				case 12/*15*/: fcemenu_loop_options(); break;
				case 13/*16*/: // done (update and write)
					config_commit();
					ret = SaveConfig(NULL);
					strcpy(menuErrorMsg, ret == 0 ? "default config saved" : "config save failed");
					UpdateScreenData();
					return 1;
				case 14: // done (update and write for current game)
					if (lastLoadedGameName[0])
					{
						config_commit();
						ret = SaveConfig(lastLoadedGameName);
						strcpy(menuErrorMsg, ret == 0 ? "game config saved" : "config save failed");
					}
					UpdateScreenData();
					return 1;
			}
		}
		if(inp & (SYMBIAN_SELECT2)) {
			config_commit();
			UpdateScreenData();
			return 0;  // done (update, no write)
		}
		if(inp & (SYMBIAN_LEFT|SYMBIAN_RIGHT|SYMBIAN_SELECT3)) { // multi choise
			switch (menu_sel) {
				case  0: int_incdec(&Settings.orientation,   (inp & SYMBIAN_LEFT) ? -1 : 1,  0,  2); break;
				case  3: int_incdec(&Settings.frameskip, (inp & SYMBIAN_LEFT) ? -1 : 1, -1, 32); break;
				case  6:
					Settings.sound_rate = sndrate_prevnext(Settings.sound_rate, inp & SYMBIAN_RIGHT);
					InitSound();					
					break;
				case  7: int_incdec(&Settings.region_force,   (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 2); break;
				case  8: {
					int hz = Settings.turbo_rate_add*60/2 >> 24;
					int_incdec(&hz, (inp & SYMBIAN_LEFT) ? -1 : 1, 1, 30);
					Settings.turbo_rate_add = (hz*2 << 24) / 60 + 1;
					break;
				}
				case  9: int_incdec(&Settings.sstate_confirm, (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 3); break;
				case  10: int_incdec(&CurrentState,            (inp & SYMBIAN_LEFT) ? -1 : 1, 0, 9); break;		
			}
		}
	}
}

// -------------- credits --------------

static void draw_menu_credits(void)
{
	char vstr[32];

	//int tl_x = 15, tl_y = 70;
	gp2x_fceu_copy_bg();

	sprintf(vstr, "GPFCE v" SYMBIAN_PORT_VERSION " rev%i", SYMBIAN_PORT_REV);
	gp2x_text_out15(10,  30, vstr);
	gp2x_text_out15(10,  40, "(c) notaz, 2007/2008");

	gp2x_text_out15(10,  70, "Based on FCE Ultra versions");
	gp2x_text_out15(10,  80, "0.81 and 0.98.1x");

	gp2x_text_out15(10, 110, "         - Credits - ");
	gp2x_text_out15(10, 130, "Bero: FCE");
	gp2x_text_out15(10, 140, "Xodnizel: FCE Ultra");
	gp2x_text_out15(10, 150, "zzhu8192: original port");
	gp2x_text_out15(10, 160, "rlyeh: minimal lib");
	gp2x_text_out15(10, 170, "Hermes, theoddbot,");
	gp2x_text_out15(10, 180, "god_at_hell: cpuctrl");
	gp2x_text_out15(10, 190, ", gamma libs  ");
	gp2x_text_out15(10, 200, "Squidge: squidgehack");

	menu_darken_text_bg();
	menu_flip();
}


// -------------- root menu --------------

static void draw_menu_root(int menu_sel)
{
	int tl_x = 30, tl_y = 126, y;
	gp2x_fceu_copy_bg();

	// error / version
	if (menuErrorMsg[0]) gp2x_text_out15(1, 229, menuErrorMsg);
	else {
		char vstr[16];
		sprintf(vstr, "v" SYMBIAN_PORT_VERSION " r%i", SYMBIAN_PORT_REV);
		gp2x_text_out15(232-strlen(vstr)*8-1, 228, vstr);
	}
	menu_darken_text_bg();
	menu_darken_reset();

	y = tl_y;
	if (fceugi) {
		gp2x_text_out15(tl_x, y,       "Resume game");
		gp2x_text_out15(tl_x, (y+=10), "Save State");
		gp2x_text_out15(tl_x, (y+=10), "Load State");
		gp2x_text_out15(tl_x, (y+=10), "Reset game");
	} else {
		y += 30;
	}
	gp2x_text_out15(tl_x, (y+=10), "Load new ROM");
	gp2x_text_out15(tl_x, (y+=10), "Options");
	gp2x_text_out15(tl_x, (y+=10), "Controls");
	gp2x_text_out15(tl_x, (y+=10), "Netplay");
	gp2x_text_out15(tl_x, (y+=10), "Credits");
	gp2x_text_out15(tl_x, (y+=10), "Exit");

	if (cheats)
		gp2x_text_out15(tl_x, (y+=10), "Cheats");

	// draw cursor
	gp2x_text_out15(tl_x - 16, tl_y + menu_sel*10, ">");

	menu_darken_text_bg();

	menu_flip();
}


static int menu_loop_root(void)
{
	int ret, menu_sel_max = 9, menu_sel_min = 4;
	static int menu_sel = 4;
	unsigned long inp = 0;

	if (fceugi) menu_sel_min = 0;
	if (cheats) menu_sel_max = 10;
	if (menu_sel < menu_sel_min || menu_sel > menu_sel_max)
		menu_sel = menu_sel_min;

	/* make sure action buttons are not pressed on entering menu */
	draw_menu_root(menu_sel);
	while (symbian_input_read() & (SYMBIAN_SELECT3|SYMBIAN_SELECT2)) usleep(50*1000);

	for (;;)
	{
		draw_menu_root(menu_sel);
		inp = wait_for_input(SYMBIAN_UP|SYMBIAN_DOWN|SYMBIAN_SELECT3|SYMBIAN_SELECT2);
		if(inp & SYMBIAN_UP  )  { menu_sel--; if (menu_sel < menu_sel_min) menu_sel = menu_sel_max; }
		if(inp & SYMBIAN_DOWN)  { menu_sel++; if (menu_sel > menu_sel_max) menu_sel = menu_sel_min; }
		if(inp &(SYMBIAN_SELECT2)){
			if (fceugi) {
				while (symbian_input_read() & SYMBIAN_SELECT2) usleep(50*1000); // wait until X is released
				Exit = 0;
				return 0;
			}
		}
		if(inp & SYMBIAN_SELECT3   )  {
			switch (menu_sel) {
				case 0: // resume game
					if (fceugi) {
						while (symbian_input_read() & SYMBIAN_SELECT3) usleep(50*1000);
						Exit = 0;
						return 0;
					}
					break;
				case 1: // save state
					if (fceugi) {
						
						FCEUI_SaveState();
						Exit = 0;
						while (symbian_input_read() & SYMBIAN_SELECT3) usleep(50*1000);
						return 0;
					}
					break;
				case 2: // load state
					if (fceugi) {
					
						FCEUI_LoadState();
						Exit = 0;
						while (symbian_input_read() & SYMBIAN_SELECT3) usleep(50*1000);
						return 0;
					}
					break;
				case 3: // reset game
					if (fceugi) {
						FCEU_DoSimpleCommand(FCEUNPCMD_RESET);
						Exit = 0;
						return 0;
					}
					break;
				case 4: // select rom
				{
					FILE *tstf;
					char *selfname;

					if ( (tstf = fopen(lastLoadedGameName, "rb")) )
					{
						fclose(tstf);
						strncpy(path_buffer, lastLoadedGameName, sizeof(path_buffer));
						path_buffer[sizeof(path_buffer)-1] = 0;
					}
					else
					{
#ifdef UIQ3
						strcpy(path_buffer, "c:\\media files\\");
#else
						strcpy(path_buffer, "c:\\data\\");
#endif
					//	getcwd(path_buffer, 256);
					}

					selfname = filesel_loop(path_buffer, lastLoadedGameName);
					if (selfname) {
						while (symbian_input_read() & SYMBIAN_SELECT3) usleep(50*1000);
						return 2;
					}
					break;
				}
				case 5: // options
					ret = menu_loop_options();
					if (ret == 1) continue; // status update
					break;
				case 6: // controls
					kc_sel_loop();
					break;
				case 7:
					network_menu_loop();
					break; // netplay
				case 8: // credits
					draw_menu_credits();
					usleep(500*1000);
					inp = wait_for_input(SYMBIAN_SELECT3|SYMBIAN_SELECT2);
					break;
				case 9: // exit
					return 1;
				case 10: // patches/gg
					patches_menu_loop();
					break;
			}
		}
		menuErrorMsg[0] = 0; // clear error msg
	}
}


extern unsigned short symbian_palette16[256];

static void menu_prepare_bg(void)
{
	menu_bg = malloc(320*240*2);
	if (menu_bg == NULL) return;

	if (fceugi)
	{
		/* raw emu frame should now be at symbian_screen */
		if (Settings.stretch != 0)
		{
#ifdef __WINS__
#else
			soft_scale((char *)symbian_screen + 32, symbian_palette16, srendline, erendline-srendline);
#endif
			if (srendline)
				memset32((int *)((char *)symbian_screen + 32), 0, srendline*320*2/4);
			memcpy(menu_bg, symbian_screen + 32, 320*240*2);
		}
		else
		{
#ifdef __WINS__
		    int cnt = 320*240;
			unsigned short int* bg = menu_bg;
			unsigned char* screen = symbian_screen;
			while(cnt)
			{
				*bg = symbian_palette16[*screen];
				screen++;
				bg++;
				cnt--;
			}
#else
			convert2RGB555(menu_bg, symbian_screen, symbian_palette16, 320*240);
#endif
		}
	}
	else
	{
		memset32((int *)menu_bg, 0, 320*240*2/4);
		//readpng(menu_bg, "background.png");
	}
}

static void menu_gfx_prepare(void)
{
	menu_prepare_bg();

	// switch bpp
	//gp2x_video_changemode(16);
#pragma message("16bit colour")
	//gp2x_video_set_offs(0);
	//gp2x_video_RGB_setscaling(320, 240);
	menu_flip();
}


int gp2x_menu_do(void)
{
	int ret;

	menu_gfx_prepare();

	ret = menu_loop_root();

	if (menu_bg) free(menu_bg);
	menu_bg = NULL;
	menuErrorMsg[0] = 0;

	return ret;
}


